/*
 *  Copyright (C) 2008 Florent Bedoiseau (electronique.fb@free.fr)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "gif_lib.h"

#include <string>
using namespace std;

int main (int argc, char **argv) {
  int i,j;
  FILE *fileOut;

  GifFileType *GIFfile;
  struct SavedImage *image;
  unsigned char *data;

  if (argc != 3 && argc != 4) {
	printf ("Syntaxe : %s <input gif> <output h file> [<name>]\n", argv[0]);
	return 0;
  }

  string name;
  if (argc == 4) {
	name = argv[3];
  }
  else {
	name = "icon";
  }

  if ((fileOut = fopen (argv[2], "w")) == NULL) {
	printf ("Unable to open output file : %s\n", argv[2]);
	return 1;
  }

  if ((GIFfile = DGifOpenFileName (argv[1])) == NULL) {
	printf ("Unable to open input file : %s\n", argv[1]);
	return 0;
  }
  
  DGifSlurp (GIFfile); // Reading image file
  
  printf ("file informations\n");
  printf ("Width : %d\n", GIFfile->SWidth);
  printf ("Height : %d\n", GIFfile->SHeight);
  printf ("ColorResolution : %d\n", GIFfile->SColorResolution);
  if (GIFfile->SColorMap != NULL) {
	printf ("Number of colors : %d\n",GIFfile->SColorMap->ColorCount);
	printf ("Bits par pixel : %d\n", GIFfile->SColorMap->BitsPerPixel);
	for (i=0; i< GIFfile->SColorMap->ColorCount; ++i) {
	  printf ("R:%d G:%d B:%d\n", GIFfile->SColorMap->Colors[i].Red, 
			  GIFfile->SColorMap->Colors[i].Green, 
			  GIFfile->SColorMap->Colors[i].Blue);
	}
  }
  else {
      printf ("No colore table\n");
  }

  printf ("Background color : %d\n", GIFfile->SBackGroundColor);
  printf ("ImageCount : %d\n", GIFfile->ImageCount);

  image = GIFfile->SavedImages; // Selecting first image

  printf ("Image informations\n");
  printf ("Left : %d\n", image->ImageDesc.Left);
  printf ("Top : %d\n", image->ImageDesc.Top);
  printf ("Width : %d\n", image->ImageDesc.Width);
  printf ("Height : %d\n", image->ImageDesc.Height);
  printf ("Interlace : %d\n", image->ImageDesc.Interlace);

  data = image->RasterBits; // Start addr 


  fprintf (fileOut, "/* File generated by %s */\n", argv[0]);
  fprintf (fileOut, "#define %s_witdh %d\n", name.c_str(),image->ImageDesc.Width);
  fprintf (fileOut, "#define %s_height %d\n", name.c_str(),image->ImageDesc.Height);
  fprintf (fileOut, "unsigned char %s [] = {\n", name.c_str());
  /* Dumping datas */
  unsigned char r, g, b;
  unsigned dataout;
  unsigned char index;
  int n = 0;
  int m = image->ImageDesc.Height * image->ImageDesc.Width;
  for (j=0; j < image->ImageDesc.Height; j++) {
	for (i=0; i <image->ImageDesc.Width; i++) {
	  index = *data; // Data corressponds to the index in the LUT 
	  r = GIFfile->SColorMap->Colors[index].Red;
	  g = GIFfile->SColorMap->Colors[index].Green;
	  b = GIFfile->SColorMap->Colors[index].Blue;
	
	  // We keep only 2 bits
	  r = r & 0xE0; // 3 bits
	  g = g & 0xE0; // 3 bits 
	  b = b & 0xC0; // 2 bits
	  dataout = r | (g >> 3) | (b >> 6); // RGB with 2 bits
	  fprintf (fileOut, "0x%02x", (unsigned char) dataout);
	  m--;
	  if (m != 0) fprintf (fileOut, ", ");

	  n++;
	  if ((n%16) == 0) {
		fprintf (fileOut, "\n");
	  }
  
	  data++;
	}
  }
  fprintf (fileOut, "\n};\n");
  fprintf (fileOut, "/* End of image */\n");
 
  if (DGifCloseFile (GIFfile) != GIF_OK) { 
	printf ("Unable to close GIF file : %s\n", argv[1]);
  }
  
  fclose (fileOut);

  return 0;
}
