/*
 *  Copyright (C) 2008 Florent Bedoiseau (electronique.fb@free.fr)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

// Circle algo : Comes from GLCD.C Stephane REY
// LUDO : iso_8859_y 8x8 font (based on console-tool package)
#include "Gfx.h"
#include <cstdlib>
#include <cstring>
#include <iostream>
using namespace std;

// Graphical section
UInt32 locateIdx=0;
UInt16 locateX=0;
UInt16 locateY=0;
UInt8 updateScreen=0;

extern UInt8 font_8x8_8859_01[]; // Declared into Font.cc
#ifdef USE_ALL_FONT
extern UInt8 font_8x8_8859_02[]; 
extern UInt8 font_8x8_8859_03[]; 
extern UInt8 font_8x8_8859_04[]; 
extern UInt8 font_8x8_8859_05[]; 
extern UInt8 font_8x8_8859_06[]; 
extern UInt8 font_8x8_8859_07[]; 
extern UInt8 font_8x8_8859_08[]; 
extern UInt8 font_8x8_8859_09[]; 
extern UInt8 font_8x8_8859_10[]; 
#endif

UInt8 *font8x8 = font_8x8_8859_01; // Default font
UInt8 *gfxScreen = 0;
gfxclip gfxClip;
UInt8 _transparentColor = 0;
#ifdef LINUX
SDL_Surface *screen = 0;
#endif



// General graphical routines
void GfxInit () {
  gfxScreen = 0;

#ifdef LINUX
  /* Initialize the SDL library */
  if( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
	fprintf(stderr, "Couldn't initialize SDL: %s\n", SDL_GetError());
	exit(1);
  }
  
  /* Clean up on exit */
  atexit(SDL_Quit);
  
  /*
   * Initialize the display in a 640x480 8-bit palettized mode,
   * requesting a software surface
   */
  screen = SDL_SetVideoMode(GFX_SCREEN_W, GFX_SCREEN_H, 8, SDL_SWSURFACE);
  if ( screen == NULL ) {
	fprintf(stderr, "Couldn't set %dx%dx8 video mode: %s\n",GFX_SCREEN_W, GFX_SCREEN_H, SDL_GetError());
	exit(1);
  }
  
  gfxScreen = new UInt8 [GFX_SCREEN_SIZE];
  GfxSetNoClip (0);

#endif
  
  GfxClear (0, 0x00);
  locateIdx = 0;
  locateY = 0;
  locateX = 0;
  font8x8 = font_8x8_8859_01; // Default font
}

void GfxQuit () {
#ifdef LINUX
  if (screen) {
    SDL_FreeSurface (screen);
    screen=0;
  }

  if (gfxScreen) {
	delete [] gfxScreen;
	gfxScreen = 0;
  }
#endif
}


/* Touch Screen functions */
void GfxGetMouseXY (UInt16 &x, UInt16 &y) {
#ifdef LINUX
  int xx, yy;
  SDL_PumpEvents();
  SDL_GetMouseState(&xx, &yy);
  x = xx;
  y = yy;
#else
  ; // TODO Not implemented
#endif
}

bool GfxGetMouseButton () {
#ifdef LINUX
  int x,y;
  SDL_PumpEvents();
  UInt8 bp = SDL_GetMouseState(&x, &y);
  if (bp & 0x01) return true;
  return false;
#else
  ; // TODO : Not implemented
  return false;
#endif
}

UInt8 GfxTouched () {
#ifdef LINUX
  SDL_Event event;
 
  int val = SDL_PollEvent(&event);
  if (!val) return 0;

  if (event.type == SDL_KEYDOWN) return 1;
  return 0;
#else
  ; // TODO : Not implemented
  return 0;
#endif
}

void GfxWaitTouched () {
#ifdef LINUX
  SDL_Event event;
  bool fin = false;
  while (fin == false) {
	SDL_WaitEvent(&event);
	
	switch (event.type) {
	case SDL_KEYDOWN:
	  fin = true;
	  break;
	case SDL_QUIT:
	  exit(0);
	  break;
	  default:
		break;
	  } 
	}
#else
  ; // TODO : Not implemented
#endif
}

// Screen only routines
void GfxLocate (UInt16 l, UInt16 c) {
  if (l >= GFX_CHAR_H) return;
  if (c >= GFX_CHAR_W) return;
  locateY = l;
  locateX = c;
  locateIdx = (8 * l * GFX_SCREEN_W) + (c*8);
}

void GfxChar (UInt8 ch, UInt8 fgColor, UInt8 bgColor) {
  UInt8 i, data;
  UInt8 line, iline;
  UInt32 adr;

  for (iline = 0; iline < FONT_HEIGHT; iline++ ) {
	if (font8x8 == 0) line = 0xFF;
	else line = font8x8[(ch * FONT_WIDTH) + iline];
	
	for (i=0; i < FONT_WIDTH; i++) {
	  data = (line << i) & 0x80;
	  adr = locateIdx + (iline * GFX_SCREEN_W) + i;
	  if (adr > GFX_SCREEN_SIZE) adr -= GFX_SCREEN_SIZE;

	  if (data) gfxScreen[adr] = fgColor; // ink color (foreground)
	  else gfxScreen[adr]= bgColor; // paper color (background)
	}
  }
  locateX++;
  if (locateX >= GFX_CHAR_W) {
	locateX=0;
	locateY++;
	if (locateY >= GFX_CHAR_H) locateY=0;
  }
  locateIdx = (8 * locateY * GFX_SCREEN_W) + (locateX * 8); // TODO : Use FONT_WIDTH and FONT_HEIGHT
  
  updateScreen = 1;
}

void GfxString (const char *dataPtr, UInt8 fgColor, UInt8 bgColor) {
  while ( *dataPtr ) {
    GfxChar(*dataPtr++, fgColor, bgColor);
  }
}



// Screen and/or bitmap
void GfxClear (gfxbitmap *bitmap, UInt8 color) {
  UInt32 i;
  UInt32 nbpix;
  UInt8 *data;
  if (!bitmap) {
	nbpix = GFX_SCREEN_SIZE;
	data = gfxScreen;
  }
  else {
	nbpix = bitmap->w * bitmap->h;
	data = bitmap->data;
  }

  for ( i = 0; i < nbpix ; i++ ){
	data[i] = color;
  }

  updateScreen = 1;
}


void GfxSelectFont (FontClass fnt) {
#ifdef USE_ALL_FONT
  switch (fnt) {
  case Iso8859_01:
	font8x8 = font_8x8_8859_01;
	break;
  case Iso8859_02:
	font8x8 = font_8x8_8859_02;
	break;
  case Iso8859_03:
	font8x8 = font_8x8_8859_03;
	break;
  case Iso8859_04:
	font8x8 = font_8x8_8859_04;
	break;
  case Iso8859_05:
	font8x8 = font_8x8_8859_05;
	break;
  case Iso8859_06:
	font8x8 = font_8x8_8859_06;
	break;
  case Iso8859_07:
	font8x8 = font_8x8_8859_07;
	break;
  case Iso8859_08:
	font8x8 = font_8x8_8859_08;
	break;
  case Iso8859_09:
	font8x8 = font_8x8_8859_09;
	break;
  case Iso8859_10:
	font8x8 = font_8x8_8859_10;
	break;
  default:
	font8x8 = font_8x8_8859_01;
	break;
  }
#endif
}

void GfxStringWidth (const char *dataPtr, UInt16 &w, UInt16 &h) {
  if (!dataPtr) { 
	w = 0;
	h = 0;
  }

  w = strlen((const char *) dataPtr) * FONT_WIDTH; // Fix-font 8x8
  h = FONT_HEIGHT;
}

void GfxStringXY (gfxbitmap *bitmap, const char *dataPtr, UInt16 x, UInt16 y, UInt8 fgColor, UInt8 bgColor) {
  UInt8 xx = x;
  UInt16 width;
  if (!bitmap) {
	width = GFX_SCREEN_W;
  }
  else {
	width = bitmap->w;
  }

  while ( *dataPtr ) {
    GfxCharXY(bitmap, *dataPtr++, xx, y, fgColor, bgColor);
	xx+=FONT_WIDTH;
	if (xx >= width) return;
  }

  updateScreen = 1;
}

void GfxCharXY (gfxbitmap *bitmap, UInt8 ch, UInt16 x, UInt16 y, UInt8 fgColor, UInt8 bgColor) {
  UInt8 i, data;
  UInt8 line, iline;
  UInt8 color;

  for (iline = 0; iline < FONT_HEIGHT; iline++ ) {
	if (font8x8 == 0) line = 0xFF;
	else line = font8x8[(ch * FONT_WIDTH) + iline];
	
	for (i=0; i < FONT_WIDTH; i++) {
	  data = (line << i) & 0x80;

	  if (data) color = fgColor; // ink color (foreground)
	  else color = bgColor; // paper color (background)

	  GfxPixel (bitmap, x+i, y+iline, color); // ink color (foreground)
	}
  }
  
  updateScreen = 1;
}

void GfxPixel (gfxbitmap *bitmap, UInt16 x, UInt16 y, UInt8 color) {
  UInt32 index;
  UInt16 w;
  UInt16 x1, x2, y1, y2;
  UInt8 *data;

  if (!bitmap) {
	w = GFX_SCREEN_W;
	x1 = gfxClip.x;
	x2 = gfxClip.x + gfxClip.w - 1;
	y1 = gfxClip.y;
	y2 = gfxClip.y + gfxClip.h - 1;
	data = gfxScreen;
  }
  else {
	w = bitmap->w;
	x1 = bitmap->clip.x;
	x2 = bitmap->clip.x + bitmap->clip.w - 1;
	y1 = bitmap->clip.y;
	y2 = bitmap->clip.y + bitmap->clip.h - 1;
	data = bitmap->data;
  }

  if (x < x1 || x > x2) return;
  if (y < y1 || y > y2) return;
 
  index = (y * w) + x;
  
  data[index] = color;

  updateScreen = 1;
}

UInt8 GfxColorPixel (gfxbitmap *bitmap, UInt16 x, UInt16 y) {
  UInt32 index;
  UInt16 w, h;
  UInt8 *data;
  if (!bitmap) {
	w = GFX_SCREEN_W;
	h = GFX_SCREEN_H;
	data = gfxScreen;
  }
  else {
	w = bitmap->w;
	h = bitmap->h;
	data = bitmap->data;
  }

  if ( x >= w ) return 0;
  if ( y >= h ) return 0;

  index = (y * w) + x;
  return data[index];
}

void GfxLine (gfxbitmap *bitmap, UInt16 x1, UInt16 y1, UInt16 x2, UInt16 y2, UInt8 color ) {
  Int32 dx, dy, stepx, stepy, fraction;

  dy = y2 - y1;
  dx = x2 - x1;

  if ( dy < 0 ) {
    dy    = -dy;
    stepy = -1;
  }
  else {
    stepy = 1;
  }

  if ( dx < 0 ) {
    dx    = -dx;
    stepx = -1;
  }
  else {
    stepx = 1;
  }

  dx <<= 1;
  dy <<= 1;

  GfxPixel(bitmap, x1, y1, color );

  if ( dx > dy ) {
    fraction = dy - (dx >> 1);
    while ( x1 != x2 ) {
      if ( fraction >= 0 ) {
	y1 += stepy;
	fraction -= dx;
      }
      x1 += stepx;
      fraction += dy;
      GfxPixel(bitmap, x1, y1, color );
    }
  }
  else {
    fraction = dx - (dy >> 1);
    while ( y1 != y2 ) {
      if ( fraction >= 0 ) {
	x1 += stepx;
	fraction -= dy;
      }
      y1 += stepy;
      fraction += dx;
      GfxPixel(bitmap, x1, y1, color );
    }
  }

  updateScreen = 1;
}

void GfxDrawPixmap (gfxbitmap *bitmap, UInt16 x, UInt16 y, UInt16 w, UInt16 h, unsigned char *pixmap) {
  UInt16 xx, yy;
  UInt8 color;
  UInt16 idx;

  if (!pixmap) return;

  for (yy = 0; yy < h; yy++) {
	for (xx = 0; xx < w; xx++) {
	  idx = (yy * w) + xx;
	  color = pixmap[idx];
	  GfxPixel (bitmap, x+xx, y+yy, color);
	}
  }

  updateScreen = 1;
}

void GfxRectangle (gfxbitmap *bitmap, UInt16 x, UInt16 y, UInt16 w, UInt16 h, UInt8 color) {
  UInt16 xx, yy, px, py;
  px = x + w;
  py = y + h;
  for (yy = y; yy < py; yy++) {
	GfxPixel (bitmap, x, yy, color);
	GfxPixel (bitmap, px-1, yy, color);
  }
  
  for (xx = x; xx < px; xx++) {
	GfxPixel (bitmap, xx, y, color);
	GfxPixel (bitmap, xx, py-1, color);
  }
}

void GfxFillRectangle (gfxbitmap *bitmap, UInt16 x, UInt16 y, UInt16 w, UInt16 h, UInt8 color) {
  UInt16 xx, yy, px, py;
  px = x + w;
  py = y + h;
  for (yy = y; yy < py; yy++) {
	  for (xx = x; xx < px; xx++) {
		GfxPixel (bitmap, xx, yy, color);
	  }
  }
}

void GfxCircle (gfxbitmap *bitmap, UInt16 x, UInt16 y, UInt16 radius, UInt8 color) {
  Int16 s16tswitch=0;
  Int16 s16y=0;
  Int16 s16x=0;
  UInt16 u8d;

  u8d = y - x;
  s16y = radius;
  s16tswitch = 3 - 2 * radius;
  while (s16x <= s16y) {
	GfxPixel(bitmap,x + s16x, y + s16y, color);
	GfxPixel(bitmap,x + s16x, y - s16y, color);
	
	GfxPixel(bitmap,x - s16x, y + s16y, color);
	GfxPixel(bitmap,x - s16x, y - s16y, color);
	
	GfxPixel(bitmap,y + s16y - u8d, y + s16x, color);
	GfxPixel(bitmap,y + s16y - u8d, y - s16x, color);
	GfxPixel(bitmap,y - s16y - u8d, y + s16x, color); 
	GfxPixel(bitmap,y - s16y - u8d, y - s16x, color);
	
	if (s16tswitch < 0) {
	  s16tswitch += (4 * s16x + 6);
	}
	else {
	  s16tswitch += (4 * (s16x - s16y) + 10);
	  s16y--;
	}
	
	s16x++;
  }
}

// TODO : Optimize
void GfxBlit (gfxbitmap *src_bitmap, gfxbitmap *dst_bitmap, UInt16 x_src, UInt16 y_src, UInt16 x_dst, UInt16 y_dst , UInt16 width, UInt16 height) {

  UInt8 color;
  UInt16 xx, yy;

  for (yy = 0; yy < height; yy++) {
	for (xx = 0; xx < width; xx++) {
	  color = GfxColorPixel (src_bitmap, x_src + xx, y_src + yy);
	  GfxPixel (dst_bitmap, x_dst + xx, y_dst + yy, color);
	}
  }

}

void GfxSetNoClip (gfxbitmap *bitmap) {
  if (!bitmap) { // Screen
	gfxClip.x = 0;
	gfxClip.y = 0;
	gfxClip.w = GFX_SCREEN_W;
	gfxClip.h = GFX_SCREEN_H;
  }
  else { // User
	bitmap->clip.x = 0;
	bitmap->clip.y = 0;
	bitmap->clip.w = bitmap->w;
	bitmap->clip.h = bitmap->h;
  }
}

void GfxSetClip (gfxbitmap *bitmap, UInt16 xc, UInt16 yc, UInt16 wc, UInt16 hc) {
  UInt16 xx, yy, ww, hh;

  if (!bitmap) { // Screen
	// X, Y
	if (xc >= GFX_SCREEN_W) xx = GFX_SCREEN_W - 1;
	else xx = xc;
	if (yc >= GFX_SCREEN_H) yy = GFX_SCREEN_H - 1;
	else yy = yc;
	
	// W, H
	if (xx + wc >=  GFX_SCREEN_W) ww = GFX_SCREEN_W - xx;
	else ww = wc;
	if (yy + hc >=  GFX_SCREEN_H) hh = GFX_SCREEN_H - yy;
	else hh = hc;

	gfxClip.x = xx;
	gfxClip.y = yy;
	gfxClip.w = ww;
	gfxClip.h = hh;
  }
  else { // User
	if (xc >= bitmap->w) xx = bitmap->w - 1;
	else xx = xc;
	if (yc >= bitmap->h) yy = bitmap->h - 1;
	else yy = yc;

	if (xx + wc >= bitmap->w) ww = bitmap->w - xx;
	else ww = wc;
	if (yy + hc >= bitmap->h) hh = bitmap->h - yy;
	else hh = hc;

	bitmap->clip.x = xx;
	bitmap->clip.y = yy;
	bitmap->clip.w = ww;
	bitmap->clip.h = hh;
  }
}

void GfxGetClip (gfxbitmap *bitmap, UInt16 &x, UInt16 &y, UInt16 &w, UInt16 &h) {
  if (!bitmap) {
	x = gfxClip.x;
	y = gfxClip.y;
	w = gfxClip.w;
	h = gfxClip.h;
  }
  else {
	x = bitmap->clip.x;
	y = bitmap->clip.y;
	w = bitmap->clip.w;
	h = bitmap->clip.h;
  }
}

UInt8 GfxColor (UInt8 r, UInt8 g, UInt8 b) {
  r = r & 0xE0; // 3 bits
  g = g & 0xE0; // 3 bits 
  b = b & 0xC0; // 2 bits
  return (r | (g >> 3) | (b >> 6)); // RGB with 2 bits
}


gfxbitmap::gfxbitmap (UInt16 width, UInt16 height) {
  w = width;
  h = height;
  data = new UInt8 [w*h];
  GfxSetNoClip (this);
}

gfxbitmap::~gfxbitmap () {
  if (data) delete [] data;
}

gfxbitmap *GfxCreateBitmap (UInt16 w, UInt16 h) {
  return new gfxbitmap (w, h);
}

gfxbitmap *GfxCreateBitmapFromPixmap (UInt16 w, UInt16 h, unsigned char *pixmap) {
  gfxbitmap *bitmap = new gfxbitmap (w, h);
  GfxDrawPixmap (bitmap, 0, 0, w, h, pixmap);
  return bitmap;
}

gfxbitmap *GfxCreateBitmapFromGif (const char *giffile) {
#ifdef GIF_SUPPORT
  if (!giffile) return 0; // KO

  GifFileType *GIFfile;
  struct SavedImage *image;
  unsigned char *data;
  UInt16 w, h, x, y;

  if ((GIFfile = DGifOpenFileName (giffile)) == 0) {
	return 0; // Error
  }
  DGifSlurp (GIFfile); // Reading image file
  image = GIFfile->SavedImages; // Selecting first image
  data = image->RasterBits; // Start addr 


  w = image->ImageDesc.Width;
  h = image->ImageDesc.Height;

  gfxbitmap *bitmap = new gfxbitmap (w, h);

  /* Dumping datas */
  UInt8 r, g, b;
  UInt8 index;

  for (y=0; y < h; y++) {
	for (x=0; x < w; x++) {
	  index = *data; // Data corressponds to the index in the LUT 
	  r = GIFfile->SColorMap->Colors[index].Red;
	  g = GIFfile->SColorMap->Colors[index].Green;
	  b = GIFfile->SColorMap->Colors[index].Blue;
	  
	  UInt8 color = GfxColor (r, g, b); // Making 3-3-2 color
	  GfxPixel (bitmap, x, y, color);

	  data++;
	}
  }

  DGifCloseFile (GIFfile);
  return bitmap;

#else
  return 0; // No GIF
#endif
}

void GfxDestroyBitmap (gfxbitmap *bitmap) {
  if (bitmap) {
	delete bitmap;
  }
}


// For simulation only
void GfxUpdate ( ) {
  if (updateScreen == 0) return;

#ifdef LINUX
  UInt8 *ptr = gfxScreen;
  UInt16 x, y;
  for (y=0; y < GFX_SCREEN_H; y++) {
	for (x=0; x < GFX_SCREEN_W; x++) {
	  UInt8 color = *ptr;
	  UInt8 r, g, b;
	  r = color & 0xE0;
	  g = (color << 3) & 0xE0;
	  b = ((color << 6) & 0xC0)| 0x20;
	  UInt32 c = SDL_MapRGB(screen->format, r, g, b);
	  putpixel (screen, x, y, c);
	  ptr++;
	}
  }
  SDL_UpdateRect(screen, 0, 0, GFX_SCREEN_W, GFX_SCREEN_H);
#endif

  updateScreen = 0;
}

#ifdef LINUX
void putpixel(SDL_Surface *screen, int x, int y, Uint32 color) {

    if ( SDL_MUSTLOCK(screen) ) {
        if ( SDL_LockSurface(screen) < 0 ) {
            return;
        }
    }
    switch (screen->format->BytesPerPixel) {
        case 1: { /* Assuming 8-bpp */
            Uint8 *bufp;

            bufp = (Uint8 *)screen->pixels + y*screen->pitch + x;
            *bufp = color;
        }
        break;

        case 2: { /* Probably 15-bpp or 16-bpp */
            Uint16 *bufp;

            bufp = (Uint16 *)screen->pixels + y*screen->pitch/2 + x;
            *bufp = color;
        }
        break;

        case 3: { /* Slow 24-bpp mode, usually not used */
		  // Not implemented
        }
        break;

        case 4: { /* Probably 32-bpp */
            Uint32 *bufp;

            bufp = (Uint32 *)screen->pixels + y*screen->pitch/4 + x;
            *bufp = color;
        }
        break;
    }
    if ( SDL_MUSTLOCK(screen) ) {
        SDL_UnlockSurface(screen);
    }
}
#endif
