/*
 *  Copyright (C) 2008 Florent Bedoiseau (electronique.fb@free.fr)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef GFX_H
#define GFX_H

#ifdef LINUX 
#include <SDL.h>  
#endif

#ifdef GIF_SUPPORT
#include "gif_lib.h"
#endif

#include "Platform.h"

#define GFX_SCREEN_W 240
#define GFX_SCREEN_H 320

#define GFX_CHAR_W (GFX_SCREEN_W / FONT_WIDTH)
#define GFX_CHAR_H (GFX_SCREEN_H / FONT_WIDTH)

#define GFX_SCREEN_SIZE (GFX_SCREEN_H * GFX_SCREEN_W)

#define FONT_WIDTH 8
#define FONT_HEIGHT 8

typedef enum {
  Iso8859_01 = 0,
  Iso8859_02,
  Iso8859_03,
  Iso8859_04,
  Iso8859_05,
  Iso8859_06,
  Iso8859_07,
  Iso8859_08,
  Iso8859_09,
  Iso8859_10
} FontClass;

class gfxclip {
 public:
  gfxclip() { x =0; y=0; w=0; h=0; }
  ~gfxclip() { }

  UInt16 x, y;
  UInt16 w, h;
};

class gfxbitmap {
 public:
  gfxbitmap (UInt16 w, UInt16 h);
  ~gfxbitmap ();

  void GfxGetWH (UInt16 &width, UInt16 &height) { width = w; height = h;}

  UInt16 w, h;
  UInt8 *data;
  gfxclip clip;

 private:
  gfxbitmap ();
};

// Graphical routines 
void GfxInit ();
void GfxQuit ();
void GfxUpdate ( ); // Simu only

#ifdef LINUX
void putpixel(SDL_Surface *surface, int x, int y, Uint32 pixel); // Simu only
#endif

void GfxGetMouseXY (UInt16 &x, UInt16 &y);
bool GfxGetMouseButton ();
UInt8 GfxTouched ();
void GfxWaitTouched ();

gfxbitmap *GfxCreateBitmap (UInt16 w, UInt16 h);
gfxbitmap *GfxCreateBitmapFromPixmap (UInt16 w, UInt16 h, unsigned char *pixmap);
gfxbitmap *GfxCreateBitmapFromGif (const char *giffile);

void GfxDestroyBitmap (gfxbitmap *bitmap);

// Screen only
void GfxLocate (UInt16 l, UInt16 c);
void GfxChar (UInt8 ch, UInt8 fgColor, UInt8 bgColor);
void GfxString (const char *str, UInt8 fgColor, UInt8 bgColor);

// Screen and/or bitmap
void GfxClear (gfxbitmap *bitmap, UInt8 color);
UInt8 GfxColor (UInt8 r, UInt8 v, UInt8 b);
void GfxSelectFont (FontClass fnt);

void GfxStringWidth (const char *str, UInt16 &w, UInt16 &h);
void GfxStringXY(gfxbitmap *bitmap, const char *str, UInt16 x, UInt16 y, UInt8 fgColor, UInt8 bgColor);
void GfxCharXY (gfxbitmap *bitmap, UInt8 ch, UInt16 x, UInt16 y, UInt8 fgColor, UInt8 bgColor);
void GfxPixel (gfxbitmap *bitmap, UInt16 x, UInt16 y, UInt8 color);
UInt8 GfxColorPixel (gfxbitmap *bitmap, UInt16 x, UInt16 y);
void GfxLine (gfxbitmap *bitmap, UInt16 x1, UInt16 y1, UInt16 x2, UInt16 y2, UInt8 color);
void GfxDrawPixmap (gfxbitmap *bitmap, UInt16 x, UInt16 y, UInt16 w, UInt16 h, unsigned char *pixmap);
void GfxRectangle (gfxbitmap *bitmap, UInt16 x, UInt16 y, UInt16 w, UInt16 h, UInt8 color);
void GfxFillRectangle (gfxbitmap *bitmap, UInt16 x, UInt16 y, UInt16 w, UInt16 h, UInt8 color);
void GfxCircle (gfxbitmap *bitmap, UInt16 x, UInt16 y, UInt16 radius, UInt8 color);

void GfxBlit (gfxbitmap *src_bitmap, gfxbitmap *dst_bitmap, UInt16 x_src, UInt16 y_src, UInt16 x_dst, UInt16 y_dst , UInt16 width, UInt16 height);
void GfxSetNoClip (gfxbitmap *bitmap);
void GfxSetClip (gfxbitmap *bitmap, UInt16 x, UInt16 y, UInt16 witdh, UInt16 height);
void GfxGetClip (gfxbitmap *bitmap, UInt16 &x, UInt16 &y, UInt16 &w, UInt16 &h); 

#endif
