/*
 *	Copyright (C) 2008 Florent Bedoiseau (electronique.fb@free.fr)
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>

#include "AW.h"
#include "AWD.h"


AW::AW (AWD *aParent, int ax, int ay, int aw, int ah) {
  int px, py;
  parent=aParent;
  if (aParent) {
    aParent->AttachObject (this);
    aParent->GetXY (px,py);
  }
  else {
    px=0;
    py=0;
  }

  x=ax+px;
  y=ay+py;
  w=aw;
  h=ah;
  CheckBounces (x,y);
  visibility=false;

  shadow_color_light=GfxColor (200,200,200);
  shadow_color_dark=GfxColor(120,120,120);
  foreground_color=GfxColor(255,255,255);
  background_color=GfxColor(20,120,200);
}

AW::~AW () {
}

void AW::SetVisibility (bool aVisibility) {
  if (visibility==aVisibility) return; // No change. Do nothing
  visibility=aVisibility;
  
  if (IsVisible()) Draw ();
  else			   UnDraw ();
}

void AW::Draw () {
  // Nothing here 
}

void AW::UnDraw () {
  // Nothing here
}

bool AW::ProcessEvent (int mouse_move, UInt16 mx, UInt16 my, 
					   int mouse_button, bool mb1, bool mb2, EVT &evt) {
#if DEBUG_EVT
  printf ("AW::ProcessEvent\n");
#endif
  return false;
}

// Event
EVT AW::Leave () {
#if DEBUG_EVT
  printf ("AW::Leave\n");
#endif
  EVT evt;
  return evt;
}
void AW::Enter () {
#if DEBUG_EVT
  printf ("AW::Enter\n");
#endif
}

void AW::Hide () {
#if DEBUG_EVT
  printf ("AW::Hide\n");
#endif 
  SetVisibility (false);
}
void AW::Show () {
#if DEBUG_EVT
  printf ("AW::Show\n");
#endif
  SetVisibility (true);
}

int AW::IsInside (int ax, int ay) const {
  if (ax<x) return 0;
  if (ay<y) return 0;
  if (ax>=x+w) return 0;
  if (ay>=y+h) return 0;
  return 1; // inside
}

int AW::Move (int ax, int ay) {
  int thex, they;
  thex=ax;
  they=ay;
  CheckBounces (thex,they); // => new x,y

  if (thex==x && they==y) return 1; // Nothing to do

  if (visibility==false) {
	x=thex;
	y=they;
  }
  else {
	UnDraw ();
	x=thex;
	y=they;
	Draw ();
  }
  return 0; // OK
}

void AW::CheckBounces (int &ax, int &ay) {
  if (parent) {
	int xx,yy,ww,hh;
	parent->GetXY (xx,yy);
	parent->GetWH (ww,hh);
	if (ax<xx) ax=xx;
	else if (ax+w >= xx+ww) ax=xx+ww-w;
	if (ay<yy) ay=yy;
	else if (ay+h >= yy+hh) ay=yy+hh-h;
  }
  else {
	if (ax<0) ax=0;
	else if (ax+w >= GFX_SCREEN_W) ax=GFX_SCREEN_W-w;
	if (ay<0) ay=0;
	else if (ay+h >= GFX_SCREEN_H) ay=GFX_SCREEN_H-h;
  }
}

EVT::EVT () : _id (0), _evtType (EVT_NONE) {
}

EVT::EVT (UInt8 id, EVT_TYPE evtType) : _id (id), _evtType (evtType) {
}
EVT::EVT(const EVT& evt) {
  _id = evt._id;
  _evtType = evt._evtType;
}

EVT& EVT::operator= (const EVT& evt) {
  if (&evt == this) return *this;
  _id = evt._id;
  _evtType = evt._evtType;
  return *this;
}

// End of file
