/*
 *  Copyright (C) 2008 Florent Bedoiseau (electronique.fb@free.fr)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

// ---------------
// AW : Top widget
// ---------------

#ifndef AW_H
#define AW_H
#include "Gfx.h"

using namespace std;

#define ENTER_CB 0
#define LEAVE_CB 1
#define HIDE_CD  2
#define SHOW_CB  3

enum EVT_TYPE { EVT_NONE = 0, EVT_PRESSED, EVT_UNPRESSED };

class EVT {
 public:
  EVT ();
  EVT (UInt8 id, EVT_TYPE evtType);
  ~EVT() { }
  EVT(const EVT& vi);
  EVT& operator= (const EVT& evt);
  bool operator== (const EVT&) const;
  //bool operator< (const EVT&) const;

  UInt8 GetId () const { return _id; }
  EVT_TYPE GetType () const { return _evtType; }

 private:
  UInt8 _id;
  EVT_TYPE _evtType;

};

class AWD;

class AW {
 public:				  
  AW (AWD *aParent, int ax, int ay, int aw, int ah);
  virtual ~AW ();

  virtual bool ProcessEvent (int mouse_move, UInt16 mx, UInt16 my, 
							 int mouse_button, bool mb1, bool mb2,
							 EVT &evt);

  virtual void Hide ();
  virtual void Show ();
  bool IsVisible () const {return visibility;}
  int IsInside (int x, int y) const;

   void SetColors (int scl, int scd, int fc, int bc) { 
    shadow_color_light=scl;
    shadow_color_dark=scd;
    foreground_color=fc;
    background_color=bc;
  }

  void GetColors (int &scl, int &scd, int &fc, int &bc) const {
    scl=shadow_color_light;
    scd=shadow_color_dark;
    fc=foreground_color;
    bc=background_color;
  }

  virtual int Move (int ax, int ay);
  void GetXY (int &ax, int &ay) const {ax=x;ay=y;}
  void GetWH (int &aw, int &ah) const {aw=w;ah=h;}
  virtual EVT Leave ();
  virtual void Enter ();

 protected:
  virtual void Draw ();
  virtual void UnDraw ();

  // Properties
  int x,y;
  int w,h;

  // Colors
  int shadow_color_light;
  int shadow_color_dark;
  int foreground_color;
  int background_color;

 private:
  void SetVisibility (bool aVisibility);
  bool visibility;
  AW *parent;
  // Returns good x,y according to the w,h and screen properties
  // or the parent properties
  void CheckBounces (int &ax, int &ay);
  
};

#endif
