/*
 *  Copyright (C) 2008 Florent Bedoiseau (electronique.fb@free.fr)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <string.h>
#include <stdio.h>
#include <iostream>
#include <deque>

#include "AWA.h"
#include "AWD.h"

int mouse_b=0;
int mouse_x=0;
int mouse_y=0;

AWA::AWA () {
  mx=0;
  my=0;
  mb1=false;
  mb2=false;
  mAWDSelected=0;
  _evtready=false;
}

AWA::~AWA () {
  // The dialogs
  list<AWD*>::iterator it=listOfAWD.begin ();
  for (; it != listOfAWD.end(); it++) {
    delete (*it);
  }
}

void AWA::AttachDialog(AWD *aAWD) {
  if (aAWD) {
    listOfAWD.push_front(aAWD);
#if DEBUG_OVERLAP
    ShowDialogOrder ();
#endif
  }
}

AWD* AWA::FindAWDUnderMouse () {
  list<AWD*>::iterator it=listOfAWD.begin ();
  for (; it != listOfAWD.end(); it++) {
    if ((*it)->IsInside (mx,my)==1) { // Only the one under the mouse
      if ((*it)->IsVisible()) return (*it);// only if visible
    }
  }
  return 0;// None found
}

bool AWA::IsCollision (const AWD *aAWDsrc, const AWD *aAWDdst) const {
  if (aAWDsrc==aAWDdst) return true; // Collision with itself

  int sx,sy,sw,sh;
  aAWDsrc->GetXY(sx,sy);
  aAWDsrc->GetWH(sw,sh);

  int dx,dy,dw,dh;
  aAWDdst->GetXY(dx,dy);
  aAWDdst->GetWH(dw,dh);

  // Src X
  int x1=sx;
  int x2=sx+sw-1;

  // Dst X
  int x3=dx;
  int x4=dx+dw-1;

  // Src Y
  int y1=sy;
  int y2=sy+sh-1;

  // Dst Y
  int y3=dy;
  int y4=dy+dh-1;

  int bx=1;
  int by=1;
  // Check in X
  if ( (x3>x2 && x4>x2) || (x3<x1 && x4<x1) ) bx=0;
  if ( (y3>y2 && y4>y2) || (y3<y1 && y4<y1) ) by=0;

  if ( (bx==1) && (by==1)) {
#if DEBUG_OVERLAP
    printf ("----------\n");
    printf ("x1:%d x2:%d y1:%d y2:%d Idsrc:%d\n", x1,x2,y1,y2, aAWDsrc->GetId());
    printf ("x3:%d x4:%d y3:%d y4:%d Iddst:%d\n", x3,x4,y3,y4, aAWDdst->GetId());
    printf ("----------\n");
#endif
    return true;
  }

  return false;
}

bool AWA::Front (AWD *aAWD) {
  if (!aAWD->IsFrontable()) return false;

  // Check if the first one is the aAWD
  list<AWD*>::iterator i=listOfAWD.begin ();
  if ((*i) == aAWD) return false; // Already in front

  // Hide all and move the
#if DEBUG_OVERLAP
  printf ("Hiding:");
#endif

  list<AWD*>::iterator ith;
  // on masque tout les visibles (TODO : seulement ceux en conflits)
  bool fin = false;
  for (ith=listOfAWD.begin (); ith != listOfAWD.end() || fin == false; ith++) {
#if DEBUG_OVERLAP
    printf ("%d ", (*ith)->GetId());
#endif

    if (IsCollision(aAWD, (*ith))== true && (*ith)->IsVisible()) {
    //if ((*ith)->IsVisible()) { // On ne masque que les visibles
        (*ith)->InternalHide ();
        (*ith)->SetShowFlag(true); // On devra le reafficher
    }

    if (aAWD == (*ith)) { // on a trouve le dialogue a mettre en avant plan
        fin = true; // on s'arretera a masquer
        ith=listOfAWD.erase(ith); // remove this element
        listOfAWD.push_front(aAWD); // Add it at the beginning
    }
  }

#if DEBUG_OVERLAP
  printf ("\nShowing:");
#endif

  // draw them in reverse order
  list<AWD*>::reverse_iterator its;
  for (its=listOfAWD.rbegin (); its != listOfAWD.rend(); its++) {
    // show all in this order
#if DEBUG_OVERLAP
    printf ("%d ", (*its)->GetId());
#endif

    if ((*its)->NeedToShow()) {
        (*its)->InternalShow ();
        (*its)->SetShowFlag(false); // On vient de le reafficher
    }
  }

#if DEBUG_OVERLAP
  printf ("\nNew order:");
  ShowDialogOrder ();
  printf ("\n--------\n");
#endif
  return true; // OK
}

void AWA::ShowDialogOrder () const {
  list<AWD*>::const_iterator it=listOfAWD.begin ();
  for (; it != listOfAWD.end(); it++) {
    printf ("%d ", (*it)->GetId());
  }
}

void AWA::MainLoop () {
  while (1) {
#ifdef LINUX
	GfxUpdate ( );
	if (GfxTouched ()) return;
#endif
    ManageDialogs (); // Manage dialogs to show and hide
    ManageMouse (); // Manage mouse events
    Process (); // user "callback" managment. Should be subclassed
  }

}

bool AWA::ManageDialogs () {
  bool toprocess = false;

  // On verifie s'il y a besoin de masquer ou montrer un dialog
  list<AWD*>::iterator ith;
  for (ith=listOfAWD.begin (); ith != listOfAWD.end(); ith++) {
      AWD *d=(AWD*) (*ith);
      if (d->NeedToShow() || d->NeedToHide()) {
          toprocess=true;
      }
  }

  if (!toprocess) return false; // Rien a faire

  list<AWD*> todraw;

  // On les masque tous (a optimiser)
  for (ith=listOfAWD.begin (); ith != listOfAWD.end(); ith++) {
      AWD *d=(AWD*) (*ith);
      if (d->IsVisible()) { // On ne masque que ceux qui sont deja visibles
          d->InternalHide ();
          if (!d->NeedToHide() ) { // On ne redessinera PAS ceux marques ToHide
              todraw.push_back(d);
          }
      }
      else {
          if (d->NeedToShow()) { // On dessira ceux invisibles mais marques ToShow
            todraw.push_back(d); // ???
          }
      }
      d->SetShowFlag(false);
      d->SetHideFlag(false);
  }

  // draw them in reverse order
  list<AWD*>::reverse_iterator its;
  for (its=todraw.rbegin (); its != todraw.rend(); its++) {
      (*its)->InternalShow ();
  }

  return true;
}

bool AWA::ManageMouse () {
  int mouse_move, mouse_button;
  if (GetMouseEvent (mouse_move, mouse_button) == false) return false; // No change. Nothing to do
  static int nselected=0;
  EVT evt;

  // Entering or leaving a Dialog
  // Only when mouse button 1 is not pressed
  if (mb1==false) {
    AWD *aAWD=FindAWDUnderMouse ();
    if (aAWD) {
      if (aAWD!=mAWDSelected) {
		if (mAWDSelected) {
		  evt = mAWDSelected->Leave ();
#if DEBUG_EVT // Button released with cursor on another dialog
		  if (evt.GetType() != EVT_NONE) {
			cerr << "#1 Un event : Id:" << evt.GetId() << " evt:" << evt.GetType() << endl;
		  }
#endif
		}
		aAWD->Enter ();
		mAWDSelected=aAWD;
		nselected=1;
      }
    }
    else {
      if (mAWDSelected) {
		evt = mAWDSelected->Leave ();
#if DEBUG_EVT // Button released with cursor on back window
		if (evt.GetType() != EVT_NONE) {
		  cerr << "#2 Un event : Id:" << evt.GetId() << " evt:" << evt.GetType() << endl;
		}
#endif
		mAWDSelected=0;
		nselected=0;
      }
    }
  }

  if (mAWDSelected) { // A dialog under the cursor
    if (mb1==true) {
      if (nselected==1) {
		Front (mAWDSelected);
		nselected=0;
      }
    }

    bool e = mAWDSelected->ProcessEvent(mouse_move, mx, my, mouse_button, mb1, mb2, evt);
#if DEBUG_EVT // Button pressed or released on same dialog
		if (e) {
		  cerr << "#3 Un event : Id:" << evt.GetId() << " evt:" << evt.GetType() << endl;
		}
#endif
  }

  if (evt.GetType() != EVT_NONE) {
	AddEvent (evt);
	return true; // New event added
  }

  return false; // No event
}

void AWA::Process () {
  EVT evt;
  if (GetEvent(evt)) {
	cerr << "Event : " << evt.GetType() << " Id : " << evt.GetId() << endl;
  }
}

// Returns 1 if there is a change
// Returns 0 if no change
bool AWA::GetMouseEvent (int &mouse_move, int &mouse_button) {
  bool status=false;
  mouse_move=0;// No move
  mouse_button=0;// No button change


  bool bb1 = GfxGetMouseButton();
  UInt16 xx,yy;
  GfxGetMouseXY (xx,yy);

  if (xx!=mx || yy!=my) {
    mx=xx;
    my=yy;
    status=true;// A change
    mouse_move=1; // Move
  }

  if (bb1!=mb1) {
    mb1=bb1;
    status=true; // A change
    mouse_button=1; // Button
  }
  return status;
}

bool AWA::GetEvent (EVT &evt) {
  if (_evtready == false) return false;

  evt = _events;
  _evtready = false;
  return true;
}

void AWA::AddEvent (const EVT &evt) {
  _events = evt;
  _evtready = true;
}
