/*
 *  Copyright (C) 2008 Florent Bedoiseau (electronique.fb@free.fr)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

// TODO
// Rendre insensible les boutons (devalides)

#include <stdio.h>
#include <iostream>

#include "AW.h"
#include "AWA.h" // Application
#include "AWRD.h" // Root Dialog
#include "AWD.h" // Dialog
#include "AWVB.h"// Base for the buttons
#include "AWB.h" // Button
#include "AWG.h" // Gauge
#include "AWL.h" // Label
#include "AWC.h" // Check button
#include "AWI.h" // Icon button

#include "logo.h"
#define ID_BUTTON_PREVIOUS 100
#define ID_BUTTON_NEXT     101
#define ID_BUTTON_CLOSE    102
#define ID_BUTTON_SHOW     103

class TemperatureApp : public AWA {
public:
    TemperatureApp();
    virtual ~TemperatureApp();

private:
    void Process ();

    // Dialogues et leurs objects

    // Root dialog
    AWD *_dialogRoot;
    AWI *_buttonS1;
    AWI *_buttonS2;

    // Temperature dialog
    AWD *_dialogTemperature;
    AWB *_buttonPrevious;
    AWB *_buttonNext;
    AWB *_buttonShow;
    AWL *_labelTitle;
    AWL *_labelFPGA [20];
    AWL *_labelC[20];
    AWL *_labelMin[20];
    AWL *_labelMax[20];
    AWL *_labelThreshold;
    AWL *_labelThresholdValue;
    AWG *_gauge;

    // Dialogue supplementaire
    AWD *_dialogHelp ;
    AWB *_buttonClose;
    float _thresholdValue;
};

TemperatureApp::TemperatureApp () {
  // Dialog principal
  _dialogRoot = new AWRD ();
  _buttonS1 = new AWI (_dialogRoot, 60,24,28,27, ID_BUTTON_CLOSE, "smilley_1.gif", "smilley_2.gif");
  _buttonS2 = new AWI (_dialogRoot, 90,24,28,27, ID_BUTTON_SHOW, "smilley_2.gif", "smilley_1.gif");

  // Dialog temperature
  _dialogTemperature = new AWD (0,0,200,200,"Temperatures");
  _buttonPrevious = new AWB (_dialogTemperature, 5, 175, 20,20, ID_BUTTON_PREVIOUS, "<");
  _buttonNext = new AWB (_dialogTemperature, 175, 175, 20,20, ID_BUTTON_NEXT, ">");
  _buttonShow = new AWB (_dialogTemperature, 35, 175, 20,20, ID_BUTTON_SHOW, "?");
  _labelTitle = new AWL (_dialogTemperature, 5, 20, 180, 8, "FPGA  Current Min  Max");
  char fpga[10];
  char tcurrent[10];
  char tmin[10];
  char tmax[10];

  UInt8 fc=GfxColor (255,255,255);
  for (int i = 0; i< 10; i++) {
      int val1=rand() % 1000;
      int val2=rand() % 1000;
      int val3=rand() % 1000;
      sprintf (fpga, "%d", i);
      sprintf (tcurrent, "%03.1f", (float) (val1/10.0));
      sprintf (tmin, "%03.1f", (float) (val2/10.0));
      sprintf (tmax, "%03.1f", (float) (val3/10.0));
      int y = ((i+1)*10)+20;

      _labelFPGA[i] = new AWL (_dialogTemperature, 5, y, 45, 8, fpga);
      _labelFPGA[i]->SetColors (0, 0, fc, GfxColor (0,0,255));
      _labelC[i]    = new AWL (_dialogTemperature, 70, y, 45, 8, tcurrent);
      _labelC[i]->SetColors (0, 0, fc, GfxColor (255,0,255));
      _labelMin[i]  = new AWL (_dialogTemperature, 115, y, 45, 8, tmin);
      _labelMin[i]->SetColors (0, 0, fc, GfxColor (0,255,255));
      _labelMax[i]   = new AWL (_dialogTemperature, 180, y, 45, 8, tmax);
      _labelMax[i]->SetColors (0, 0, fc, GfxColor (255,0,0));
  }

  _labelThreshold = new AWL (_dialogTemperature, 5, 160, 80, 8, "Current");
  char buf[20];
  _thresholdValue = 10.0;
  sprintf (buf, "%02.1f", _thresholdValue);
  _labelThresholdValue = new AWL (_dialogTemperature, 160, 160, 30, 8, buf);
  _gauge = new AWG (_dialogTemperature, 70, 160, 80, 8, false, 0.0, 50.0);
  _gauge->SetValue (10.0);

  // Dialog Help
  _dialogHelp =  new AWD (20,220,210, 60,"Help");
  _buttonClose = new AWB (_dialogHelp ,5, 30,200,20, ID_BUTTON_CLOSE,"CLOSE");

  // For managment
  AttachDialog (_dialogRoot);
  AttachDialog (_dialogTemperature); // Needded to process events
  AttachDialog (_dialogHelp);

  // Showing de main ones
  _dialogRoot->Show ();
  _dialogTemperature->Show (); // Let's show it !
}

TemperatureApp::~TemperatureApp () {
}

void TemperatureApp::Process () {
  EVT evt;
  if (GetEvent(evt)) {
	int id = evt.GetId();
    if (evt.GetType() != EVT_PRESSED) return;

	// Incremente/Decremente en fonction du bouton appuye
	if (id == ID_BUTTON_PREVIOUS || id == ID_BUTTON_NEXT) {
	    char buf[20];

	    if (id == ID_BUTTON_PREVIOUS) {
            if (_thresholdValue>0) _thresholdValue--;
	    }
	    else {
            if (_thresholdValue<100) _thresholdValue++;
	    }
	    sprintf (buf, "%02.1f", _thresholdValue);
	    _labelThresholdValue->SetText(buf);
	    _labelC[2]->SetText(buf);
	    _gauge->SetValue(_thresholdValue);
	    return;
	}

	// Affiche le dialog Help
	if (id == ID_BUTTON_SHOW && !_dialogHelp->IsVisible()) {
	    _dialogHelp->Show();
	    return;
	}

	// Ferme le dialog Help
    if (id == ID_BUTTON_CLOSE) {
	    _dialogHelp->Hide();
	    return;
	}
  }
}

int main () {
  GfxInit(); // Graphic initialization
  GfxClear(0, GfxColor(60, 120, 20));
  TemperatureApp app; // All the application
  app.MainLoop ();
  return 0;
}
