/*
 *  Copyright (C) 2008 Florent Bedoiseau (electronique.fb@free.fr)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include "WdtCheckBox.h"

Wdt_CHECKBOX * WdtCheckBoxNew (UInt16 x, UInt16 y, UInt16 size, UInt8 fgcolor, UInt8 bgcolor) {
  Wdt_CHECKBOX *p = (Wdt_CHECKBOX *) calloc (1, sizeof (Wdt_CHECKBOX));
  if (p) {
    p->x = x;
    p->y = y;
	p->_fgcolor = fgcolor;
	p->_bgcolor = bgcolor;

    if (size<8) p->size = 8;
    else p->size = size;
    p->toredraw = true;
    p->checked = false; 
    p->frozen = false; 
    p->focus = false;
  }
  return p;
}

void WdtCheckBoxDelete (Wdt_CHECKBOX *aWdtCheckBox) {
  if (!aWdtCheckBox) {
#ifdef DEBUG
    printf ("WdtCheckBoxDelete : WdtCheckBox = NULL\n");
#endif
    return;
  }

  free (aWdtCheckBox);
}

void WdtCheckBoxDraw (Wdt_CHECKBOX *aWdtCheckBox, bool force) {
  UInt16 x, y;
  if (!aWdtCheckBox) {
#ifdef DEBUG
    printf ("WdtCheckBoxDraw : WdtCheckBox = NULL\n");
#endif
    return;
  }

  if ((force == false) && (aWdtCheckBox->toredraw == false)) // Conditionned by toredraw flag
    return; // Nothing to do

  // Erase the area
  GfxFillRectangle (aWdtCheckBox->x, aWdtCheckBox->y,
					aWdtCheckBox->size,
					aWdtCheckBox->size,
					aWdtCheckBox->_bgcolor);
 
  // We choose the right image according to the focus
  if (aWdtCheckBox->focus == 1) {
    // Draw the focus rectangle

    GfxRectangle ( aWdtCheckBox->x,
				   aWdtCheckBox->y,
				   aWdtCheckBox->size,
				   aWdtCheckBox->size,
				   aWdtCheckBox->_fgcolor);
	
  }
  else {
    // Do not draw the focus rectangle
  }
  
  // Draw the cross bar
  if (aWdtCheckBox->checked) {
    GfxLine (aWdtCheckBox->x+2,
			 aWdtCheckBox->y+2,
			 aWdtCheckBox->x + (aWdtCheckBox->size - 3),
			 aWdtCheckBox->y + (aWdtCheckBox->size - 3),
			 aWdtCheckBox->_fgcolor);
    
    GfxLine (aWdtCheckBox->x+2,
			 aWdtCheckBox->y + (aWdtCheckBox->size - 3),
			 aWdtCheckBox->x + (aWdtCheckBox->size - 3),
			 aWdtCheckBox->y+2,
			 aWdtCheckBox->_fgcolor);
  }

  if (aWdtCheckBox->frozen) {
    // Draw a grey box and sign here
    for (x=1; x< aWdtCheckBox->size-2; x+=2) {
	  GfxPixel (aWdtCheckBox->x + x,
				aWdtCheckBox->y+1, 
				aWdtCheckBox->_fgcolor);

      GfxPixel (aWdtCheckBox->x + x,
				aWdtCheckBox->y+ (aWdtCheckBox->size - 2),
				aWdtCheckBox->_fgcolor);
    }

    for (y=1; y< aWdtCheckBox->size-2; y+=2){
      GfxPixel (aWdtCheckBox->x+1,
				aWdtCheckBox->y+y,
				aWdtCheckBox->_fgcolor);

      GfxPixel (aWdtCheckBox->x+(aWdtCheckBox->size - 2),
				aWdtCheckBox->y+y,
				aWdtCheckBox->_fgcolor);
    }
    
  }
  else {

    GfxRectangle (aWdtCheckBox->x+1,
				  aWdtCheckBox->y+1,
				  aWdtCheckBox->size-2,
				  aWdtCheckBox->size-2,
				  aWdtCheckBox->_fgcolor);

  }

  aWdtCheckBox->toredraw = false;
}

void WdtCheckBoxSetFocus (Wdt_CHECKBOX *aWdtCheckBox, bool focus) {
  if (!aWdtCheckBox) {
#ifdef DEBUG
    printf ("WdtCheckBoxSelect : WdtCheckBox : NULL\n");
#endif
    return;
  }

  if (aWdtCheckBox->focus == focus) return; 

  if (aWdtCheckBox->frozen) return; // Should not change the focus

  aWdtCheckBox->focus = focus;
  aWdtCheckBox->toredraw = true; // Value has changed. Need to redraw
}

bool WdtCheckBoxGetFocus (Wdt_CHECKBOX *aWdtCheckBox) {
  if (!aWdtCheckBox) {
#ifdef DEBUG
    printf ("WdtCheckBoxGetFocus : WdtCheckBox : NULL\n");
#endif
    return false;
  }

  return aWdtCheckBox->focus;
}

bool WdtCheckBoxGetFrozen (Wdt_CHECKBOX *aWdtCheckBox) {
  if (!aWdtCheckBox) {
#ifdef DEBUG
    printf ("WdtCheckBoxGetFrozen : WdtCheckBox : NULL\n");
#endif
    return false;
  }

  return aWdtCheckBox->frozen;
}

void WdtCheckBoxSetFrozen (Wdt_CHECKBOX *aWdtCheckBox, bool frozen) {
  if (!aWdtCheckBox) {
#ifdef DEBUG
    printf ("WdtCheckBoxSetFrozen : WdtCheckBox : NULL\n");
#endif
    return;
  }

  if (aWdtCheckBox->frozen == frozen) return;

  aWdtCheckBox->frozen = frozen;
  aWdtCheckBox->toredraw = true; // Value has changed. Need to redraw
}

bool WdtCheckBoxGetChecked (Wdt_CHECKBOX *aWdtCheckBox) {
  if (!aWdtCheckBox) {
#ifdef DEBUG
    printf ("WdtCheckBoxGetChecked : WdtCheckBox : NULL\n");
#endif
    return false;
  }

  return aWdtCheckBox->checked;
}

void WdtCheckBoxSetChecked (Wdt_CHECKBOX *aWdtCheckBox, bool checked) {
  if (!aWdtCheckBox) {
#ifdef DEBUG
    printf ("WdtCheckBoxSetChecked : WdtCheckBox : NULL\n");
#endif
    return;
  }

  if (aWdtCheckBox->checked == checked) return; 

  if (aWdtCheckBox->frozen) return; // Should not change the check

  aWdtCheckBox->checked = checked;
  aWdtCheckBox->toredraw = true; // Value has changed. Need to redraw
}
// End of file
