/*
 *  Copyright (C) 2008 Florent Bedoiseau (electronique.fb@free.fr)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include "WdtFrame.h"

Wdt_FRAME * WdtFrameNew (UInt16 x, UInt16 y, UInt16 w, UInt16 h, UInt8 fgcolor, UInt8 bgcolor) {
  Wdt_FRAME *p = (Wdt_FRAME *) calloc (1, sizeof (Wdt_FRAME));
  if (p) {
    p->x = x;
    p->y = y;
    p->w = w;
    p->h = h;
	p->_fgcolor = fgcolor;
	p->_bgcolor = bgcolor;

    p->visibility = true; // Visible
    p->toredraw = true;
  }
  return p;
}
void WdtFrameDelete (Wdt_FRAME *aWdtFrame) {
  if (!aWdtFrame) {
#ifdef DEBUG
    printf ("WdtFrameDelete : WdtFrame = NULL\n");
#endif
    return;
  }

  free (aWdtFrame);
}

void WdtFrameDraw (Wdt_FRAME *aWdtFrame) {
  if (!aWdtFrame) {
#ifdef DEBUG
    printf ("WdtFrameDraw : WdtFrame NULL\n");
#endif
    return;
  }

  if (aWdtFrame->toredraw == 0) // Conditionned by toredraw flag
	return; // Nothing to do

  UInt8 color;
  if (aWdtFrame->visibility == 0)
    color = aWdtFrame->_bgcolor;
  else
    color = aWdtFrame->_fgcolor;

  GfxRectangle (aWdtFrame->x, aWdtFrame->y, aWdtFrame->w, aWdtFrame->h, color);

  aWdtFrame->toredraw = 0;
}

bool WdtFrameGetVisibility (Wdt_FRAME *aWdtFrame) {
  if (!aWdtFrame) {
    printf ("WdtFrameGetVisibility : WdtFrame : NULL\n");
    return 0;
  }
  return aWdtFrame->visibility;
}

void WdtFrameSetVisibility (Wdt_FRAME *aWdtFrame, bool visibility) {
  if (!aWdtFrame) {
    printf ("WdtFrameSetVisibility : WdtFrame : NULL\n");
    return;
  }

  aWdtFrame->visibility = visibility;
  aWdtFrame->toredraw = 1;
  WdtFrameDraw(aWdtFrame);
}

// End of file
