// IHMFsm.c

#include "WdtFsm.h"
#include "WdtKey.h"

extern IHM_FSM_STATE fsm_states []; // State array

static FSM_STATE_ID currentstate;

void IHMFsmInit () {
  currentstate = FSM_INIT_STATE;
}


FSM_STATE_ID IHMFsmGetState () {
  return currentstate;
}

int IHMFsmSetState (FSM_STATE_ID aState) {
  if (aState >= MAX_FSM_STATES) return 1; 

  currentstate = aState;
  return 0; 
} 

int IHMFsmGetNextState (int key, FSM_STATE_ID *aState) {
   FSM_STATE_ID tempstate;
  int statechanged=0;

  if (currentstate >= MAX_FSM_STATES) {
    *aState= None;
    currentstate=None;
    return 0; // No change
  }

  if (fsm_states[currentstate].bypass != 0) {
    *aState = currentstate;
    // We "simulate" a Change. We skip the key check bellow
    return 1;
  }

  switch (key) {
	
  case WDT_KEY_ESC:
    tempstate=fsm_states[currentstate].esc;
    if (tempstate != None) {
      currentstate=tempstate;
      *aState=currentstate;
      statechanged=1;
    }
    break;

  case WDT_KEY_ENTER:
    tempstate=fsm_states[currentstate].enter;
    if (tempstate != None) {
      currentstate=tempstate;
      *aState=currentstate;
      statechanged=1;
    }
    break;

  case WDT_KEY_LEFT:
    tempstate=fsm_states[currentstate].left;
    if (tempstate != None) {
      currentstate=tempstate;
      *aState=currentstate;
      statechanged=1;
    }
    break;

  case WDT_KEY_RIGHT:
    tempstate=fsm_states[currentstate].right;
    if (tempstate != None) {
      currentstate=tempstate;
      *aState=currentstate;
      statechanged=1;
    }
    break;

  case WDT_KEY_UP:
    tempstate=fsm_states[currentstate].up;
    if (tempstate != None) {
      currentstate=tempstate;
      *aState=currentstate;
      statechanged=1;
    }
    break;

  case WDT_KEY_DOWN:
    tempstate=fsm_states[currentstate].down;
    if (tempstate != None) {
      currentstate=tempstate;
      *aState=currentstate;
      statechanged=1;
    }
    break;


  default:
    statechanged=0; // No change
    break;
  }

  return statechanged;
}


// End of file
