/*
 *  Copyright (C) 2008 Florent Bedoiseau (electronique.fb@free.fr)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include "WdtIcon.h"

Wdt_ICON * WdtIconNew (UInt16 x, UInt16 y, UInt16 w, UInt16 h, unsigned char *unsel_image, unsigned char *sel_image) {
  Wdt_ICON *p = (Wdt_ICON *) calloc (1, sizeof (Wdt_ICON));
  if (p) {
    p->x = x;
    p->y = y;
	p->w = w;
	p->h = h;
    p->unselected_image = unsel_image;
    p->selected_image = sel_image;
    p->toredraw = true; 
	p->visibility = true; // Visible
    p->selection = false; // Unselected (unsel_image by defautlt)

    p->blink_icon = 0;
    p->blink_period = 0;
    p->blink_onoff = false; // No blinking now
    p->blink_current = 0;
    p->blink_direction = false;
  }
  return p;
}

void WdtIconDelete (Wdt_ICON *aWdtIcon) {
  if (!aWdtIcon) {
#ifdef DEBUG
    printf ("WdtIconDelete : WdtIcon = NULL\n");
#endif
    return;
  }

  // Do not need to destroy images (static)
  free (aWdtIcon);
}

void WdtIconDraw (Wdt_ICON *aWdtIcon, bool force) {
  unsigned char *anImage = 0;
  if (!aWdtIcon) {
#ifdef DEBUG
    printf ("WdtIconDraw : WdtIcon = NULL\n");
#endif
    return;
  }

  if (aWdtIcon->visibility == false) return; // Nothing to do

  // BLINK is ON
  if (aWdtIcon->blink_onoff == true) {
    if (aWdtIcon->blink_direction == false) { // Inc
      aWdtIcon->blink_current++;
      if (aWdtIcon->blink_current > aWdtIcon->blink_period) {
        aWdtIcon->blink_current = aWdtIcon->blink_period;
        aWdtIcon->blink_direction = true;
        aWdtIcon->toredraw = true; // Value has changed. Need to redraw
        anImage = aWdtIcon->blink_icon;
      }
      else {
        // We choose the normal image
        if (aWdtIcon->selection == false) anImage = aWdtIcon->unselected_image; // Not selected
        else anImage = aWdtIcon->selected_image; // Selected
      }
    }
    else { // Dec
      aWdtIcon->blink_current--;
      if (aWdtIcon->blink_current < 0) {
        aWdtIcon->blink_direction = false;
        aWdtIcon->blink_current = 0;
        aWdtIcon->toredraw = 1; // Value has changed. Need to redraw
        // We take the normal image
        if (aWdtIcon->selection == false) anImage = aWdtIcon->unselected_image; // Not selected
        else anImage = aWdtIcon->selected_image; // Selected
      }
      else {
        anImage = aWdtIcon->blink_icon;
      }
    }
  }
  else { // BLINK is OFF
    // We choose the right image according to the selection
    if (aWdtIcon->selection == 0) anImage = aWdtIcon->unselected_image; // Not selected
    else anImage = aWdtIcon->selected_image; // Selected
  }

  if ((force == false) && (aWdtIcon->toredraw == false)) // Conditionned by toredraw flag
    return; // Nothing to do

  // OK, we draw what we have to draw ...
  GfxPixmap (aWdtIcon->x,
			 aWdtIcon->y,
			 aWdtIcon->w,
			 aWdtIcon->h,
			 anImage);

  aWdtIcon->toredraw = false;
}

void WdtIconSetBlinkParameters (Wdt_ICON *aWdtIcon, unsigned char *blink_icon, UInt8 blink_period) {
  if (!aWdtIcon) {
#ifdef DEBUG
    printf ("WdtIconSetBlinkParameters : WdtIcon : NULL\n");
#endif
    return;
  }

  if ((blink_icon == aWdtIcon->blink_icon) && (blink_period==aWdtIcon->blink_period)) return; // Nothing to do

  aWdtIcon->blink_icon = blink_icon;
  aWdtIcon->blink_period = blink_period;
  aWdtIcon->blink_onoff = false; // Off
  aWdtIcon->blink_current = 0;
  aWdtIcon->blink_direction = false;
  aWdtIcon->toredraw = true; // Need to redraw
}

void WdtIconSetBlinkOnOff (Wdt_ICON *aWdtIcon, bool onoff) {
  if (!aWdtIcon) {
#ifdef DEBUG
    printf ("WdtIconSetBlinkOnOff : WdtIcon : NULL\n");
#endif
    return;
  }

  if (aWdtIcon->blink_icon == 0) return; // No icon, no effect

  if (aWdtIcon->blink_onoff == onoff) return; // Nothing to do

  aWdtIcon->blink_onoff = onoff;
  aWdtIcon->toredraw = true; // Value has changed. Need to redraw
}

bool WdtIconGetBlinkOnOff (Wdt_ICON *aWdtIcon) {
  if (!aWdtIcon) {
#ifdef DEBUG
    printf ("WdtIconGetBlinkOnOff : WdtIcon : NULL\n");
#endif
    return false;
  }
  return (aWdtIcon->blink_onoff);
}

void WdtIconSetSelection (Wdt_ICON *aWdtIcon, bool selection) {
  if (!aWdtIcon) {
#ifdef DEBUG
    printf ("WdtIconSetSelection : WdtIcon : NULL\n");
#endif
    return;
  }

  if (aWdtIcon->selected_image == 0) return; // Nothing to do
  if (selection == aWdtIcon->selection) return; // Nothing to do
  
  aWdtIcon->selection = selection;
  aWdtIcon->toredraw = true; // Value has changed. Need to redraw
}

bool WdtIconGetSelection (Wdt_ICON *aWdtIcon) {
  if (!aWdtIcon) {
#ifdef DEBUG
    printf ("WdtIconGetSelection : WdtIcon : NULL\n");
#endif
    return false;
  }

  return aWdtIcon->selection;
}

bool WdtIconGetVisibility (Wdt_ICON *aWdtIcon) {
  if (!aWdtIcon) {
#ifdef DEBUG
    printf ("WdtIconGetVisibility : WdtIcon : NULL\n");
#endif
    return false;
  }

  return aWdtIcon->visibility;
}

void WdtIconSetVisibility (Wdt_ICON *aWdtIcon, bool visibility) {
  if (!aWdtIcon) {
#ifdef DEBUG
    printf ("WdtIconSetVisibility : WdtIcon : NULL\n");
#endif
    return;
  }

  if (aWdtIcon->visibility == visibility) return; // Nothing to do

  aWdtIcon->visibility = visibility;
  aWdtIcon->toredraw = true; // Value has changed. Need to redraw
}

void WdtIconSetImage (Wdt_ICON *aWdtIcon, UInt16 w, UInt16 h, unsigned char *unsel_image, unsigned char *sel_image) {
  if (!aWdtIcon) {
#ifdef DEBUG
    printf ("WdtIconSetImage : WdtIcon : NULL\n");
#endif
    return;
  }

  if ((aWdtIcon->selected_image == sel_image) && (aWdtIcon->unselected_image == unsel_image)) return; // Nothing to do

  aWdtIcon->selected_image = sel_image;
  aWdtIcon->unselected_image = unsel_image;
  aWdtIcon->w = w;
  aWdtIcon->h = h;
  
  aWdtIcon->toredraw = true; // Value has changed. Need to redraw
}


// End of file
