// WdtPanel.c

#include "WdtPanel.h"
#include "WdtImage.h"
#include "WdtLabel.h"
#include "WdtFsm.h"



#define  MSGBOX_X       250
#define  MSGBOX_Y        40
#define  MSGBOX_WIDTH   400
#define  MSGBOX_HEIGHT  150

// PANEL_P1
#define BLOCK1      100
#define BLOCK2      BLOCK1+100
#define BLOCK3      BLOCK2+100
#define BLOCK4      BLOCK3+100

void Compute_P1_BLOCK(void);
int current_P2_sheet = 0;
int current_P3_sheet = 0;

Wdt_PANEL panel [MAX_PANELS];
int selected_panel = MAX_PANELS;

void WdtPanelInit (GR_DRAW_ID wid, GR_GC_ID gc, GR_GC_ID gc_progress) {

  selected_panel = INIT_PANEL;

  // Create and init all the different panels here
  WdtPanelInit_P1 ();
  WdtPanelInit_P2 ();
  WdtPanelInit_P3 ();
  WdtPanelInit_A0 ();
  WdtPanelInit_A1 ();
  WdtPanelInit_A2 ();
  WdtPanelInit_A3 ();
  WdtPanelInit_A4 ();
  WdtPanelInit_A5 ();
  WdtPanelInit_A6 ();
  WdtPanelInit_P11 ();
  WdtPanelInit_P12 ();
  WdtPanelInit_P13 ();
  WdtPanelInit_P14 ();
  WdtPanelInit_P15 (); 
  WdtPanelInit_P22 ();
}

void WdtPanelDraw () {
  int i;
  PANEL_ID aPanel;

  // We select a specific panel 
  // We clear the screen 
  // We draw all the objects
  if (selected_panel >= MAX_PANELS) {
#ifdef DEBUG 
    printf ("ActivatePanel : No selected panel\n"); 
#endif
    return;// KO 
  } 

  GrClearWindow (panelwid, 0); // NO EXPOSURE evt generated
  
  aPanel = selected_panel;

  // Draw Icons
  for (i=0; i < MAX_PANEL_ICONES; i++) WdtIconDraw (panel[aPanel].icones[i], 1);
 
  // Draw Frames
  for (i=0; i < MAX_PANEL_FRAMES; i++) WdtFrameDraw (panel[aPanel].frames[i]);
   
  // Draw Frames
  for (i=0; i < MAX_PANEL_PROGRESS; i++) WdtProgressDraw (panel[aPanel].progress[i], 1); // 1 : Force to draw

  // Draw labels
  for (i=0; i < MAX_PANEL_LABELS; i++) WdtLabelDraw (panel[aPanel].labels[i], 1); // 1 : Force to draw

  // Draw checkboxes
  for (i=0; i < MAX_PANEL_CHECKBOXES; i++) WdtCheckBoxDraw (panel[aPanel].checkboxes[i], 1); // 1 : Force to draw

  // Draw textfields
  for (i=0; i < MAX_PANEL_TEXTFIELDS; i++) WdtTextFieldDraw (panel[aPanel].textfields[i], 1); // 1 : Force to draw

} 

void WdtPanelUpdate () 
{
  int i;
  int aPanel;
  float chariot, levage, moment, charge;
  static float chariot_sv = 0;
  static float levage_sv = 0;
  static float moment_sv = 0;
  static float charge_sv = 0;
  static int counter = 0;
  static int sens = 1;
  float TOR_test;
//  float P12_val1, P12_val2;
//  static float P12_val1_sv = 0;
//  static float P12_val2_sv = 0;
//  char item[10];

  // We select a specific panel 
  // We draw all the objects only if needed
  if (selected_panel >= MAX_PANELS) {
#ifdef DEBUG
    printf ("WdtPanelUpdate : No selected panel\n"); 
#endif
    return;// KO 
  } 

  aPanel = selected_panel;
  if (counter > 100)
    sens = -1;
  if (counter < -100)
    sens = 1;
  counter = counter + sens;
  
  if (aPanel == PANEL_P1)
  {
    WdtProgressSetValidity(panel[PANEL_P1].progress[0], 1);
    WdtProgressSetValidity(panel[PANEL_P1].progress[1], 1);
    WdtProgressSetValidity(panel[PANEL_P1].progress[2], 1);
    WdtProgressSetValidity(panel[PANEL_P1].progress[3], 1);
#ifndef SIMU
    if (GetOimValueByType(OIM_CHARIOT, &chariot) == -1)
      WdtProgressSetValidity(panel[PANEL_P1].progress[0], 0);
    if (GetOimValueByType(OIM_LEVAGE, &levage) == -1)
      WdtProgressSetValidity(panel[PANEL_P1].progress[1], 0);
    if (GetOimValueByType(OIM_MOMENT, &moment) == -1)
      WdtProgressSetValidity(panel[PANEL_P1].progress[2], 0);
    if (GetOimValueByType(OIM_CHARGE, &charge) == -1)
      WdtProgressSetValidity(panel[PANEL_P1].progress[3], 0);
    charge = charge / 1000;

    WdtCheckTOR(PANEL_P1, P1_I5, IMG_GRUE_GD_BLKD, KEY_ITEM_IN_TOR_MECA_CHAR_AV);
    WdtCheckTOR(PANEL_P1, P1_I5, IMG_GRUE_GD_BLKG, KEY_ITEM_IN_TOR_MECA_CHAR_AR);
    WdtCheckTOR(PANEL_P1, P1_I6, IMG_GRUE_HB_BLKH, KEY_ITEM_IN_TOR_MECA_LEV_MNT);
    WdtCheckTOR(PANEL_P1, P1_I6, IMG_GRUE_HB_BLKB, KEY_ITEM_IN_TOR_MECA_LEV_DSC);
    WdtCheckTOR(PANEL_P1, P1_I7, IMG_ELEVATOR_BLK, KEY_ITEM_IN_TOR_SECU_MOMENT);
    WdtCheckTOR(PANEL_P1, P1_I8, IMG_POIDS_BLK   , KEY_ITEM_IN_TOR_SECU_CHARGE);

#else
//    printf("levage = %f, chariot = %f, moment = %f, charge = %f\n", levage, chariot, moment, charge);
    chariot = counter;
    levage  = 12;
    moment  = 73;
    charge  = 2.32;
    if (counter == 40)
      WdtIconSetBlinkParameters (panel[PANEL_P1].icones[P1_I5], IMG_GRUE_GD_BLKD, 1);
    if (counter > 40)
      WdtIconSetBlinkOnOff (panel[PANEL_P1].icones[P1_I5], 1);
#endif
    if (chariot != chariot_sv)
      WdtProgressSetValue (panel[PANEL_P1].progress[0], chariot);
    if (levage != levage_sv)
      WdtProgressSetValue (panel[PANEL_P1].progress[1], levage);
    if (moment_sv != moment)
      WdtProgressSetValue (panel[PANEL_P1].progress[2], moment);
    if (charge != charge_sv)
      WdtProgressSetValue (panel[PANEL_P1].progress[3], charge);
    chariot_sv = chariot;
    levage_sv  = levage;
    moment_sv = moment;
    charge_sv = charge;
  }

  if ((aPanel == PANEL_P12) || (aPanel == PANEL_P13) || (aPanel == PANEL_P14) || (aPanel == PANEL_P15))
  {
    if (aPanel == PANEL_P12)
      WdtPanelSetP1X(KEY_ITEM_IN_ANA_DIST_CHARIOT, PANEL_P12);
    if (aPanel == PANEL_P13)
      WdtPanelSetP1X(KEY_ITEM_IN_ANA_DIST_LEVAGE, PANEL_P13);
    if (aPanel == PANEL_P14)
      WdtPanelSetP1X(KEY_ITEM_IN_ANA_MOMENT, PANEL_P14);
    if (aPanel == PANEL_P15)
      WdtPanelSetP1X(KEY_ITEM_IN_ANA_POID_CHARGE, PANEL_P15);
  }

  // Redraw only widgets that can change dynamicaly
  // Frames & Labels for the moment
 // Draw Icons
  for (i=0; i < MAX_PANEL_ICONES; i++) WdtIconDraw (panel[aPanel].icones[i], 0);
  
  // Draw Frames
  for (i=0; i < MAX_PANEL_PROGRESS; i++) WdtProgressDraw (panel[aPanel].progress[i], 0);

  // Draw label
  // Time update works for any panel selected [time has to be label 2 and 3 !!)
  if ((aPanel == PANEL_P1) || (aPanel == PANEL_P2) || (aPanel == PANEL_P3))
  {
    time(&tm);
    memcpy(&tm_time, localtime(&tm), sizeof(tm_time));
    sprintf(HM_string, "%02d:%02d", tm_time.tm_hour, tm_time.tm_min);
    sprintf(DATE_string, "%02d/%02d/%02d", tm_time.tm_mday, tm_time.tm_mon+1, tm_time.tm_year-100);
    WdtLabelSetString(panel[aPanel].labels[P1_L2], HM_string);
    WdtLabelSetString(panel[aPanel].labels[P1_L3], DATE_string);
    WdtLabelSetString(panel[aPanel].labels[P2_L2], HM_string);
    WdtLabelSetString(panel[aPanel].labels[P2_L3], DATE_string);
    WdtLabelSetString(panel[aPanel].labels[P3_L2], HM_string);
    WdtLabelSetString(panel[aPanel].labels[P3_L3], DATE_string);
  }

//  if (aPanel == PANEL_P3) WdtPanelChangeValues_P3 ();

  // Draw Labels
  for (i = 0; i < MAX_PANEL_LABELS; i++)     WdtLabelDraw (panel[aPanel].labels[i], 0);

  // Draw checkboxes
  for (i = 0; i < MAX_PANEL_CHECKBOXES; i++) WdtCheckBoxDraw (panel[aPanel].checkboxes[i], 0); // 1 : Force to draw

  // Draw textfields
  for (i = 0; i < MAX_PANEL_TEXTFIELDS; i++) WdtTextFieldDraw (panel[aPanel].textfields[i], 0); // 1 : Force to draw

  // Draw Panels
  //for (i = 0; i < MAX_PANEL_FRAMES; i++) WdtFrameDraw(panel[aPanel].frames[i]);
} 

void WdtPanelSetP1X(unsigned int key, unsigned int Panel)
{
    float ftemp1, ftemp2;
    static float ftemp1_sv = 0;
    static float ftemp2_sv = 0;
    char item[10];
    static count = 0;
#ifndef SIMU
    if (Panel == PANEL_P12)
    {
      WdtCheckTOR(PANEL_P12, P12_I1, IMG_GRUE_GD_BLKD, KEY_ITEM_IN_TOR_MECA_CHAR_AV);
      WdtCheckTOR(PANEL_P12, P12_I1, IMG_GRUE_GD_BLKG, KEY_ITEM_IN_TOR_MECA_CHAR_AR);
    }
    if (Panel == PANEL_P13)
    {
      WdtCheckTOR(PANEL_P13, P13_I1, IMG_GRUE_HB_BLKH, KEY_ITEM_IN_TOR_MECA_LEV_MNT);
      WdtCheckTOR(PANEL_P13, P13_I1, IMG_GRUE_HB_BLKB, KEY_ITEM_IN_TOR_MECA_LEV_DSC);
    }
    if (Panel == PANEL_P14)
      WdtCheckTOR(PANEL_P14, P14_I1, IMG_ELEVATOR_BLK, KEY_ITEM_IN_TOR_SECU_MOMENT);
    if (Panel == PANEL_P15)
      WdtCheckTOR(PANEL_P15, P15_I1, IMG_POIDS_BLK   , KEY_ITEM_IN_TOR_SECU_CHARGE);
#endif
#ifdef SIMU
    count++;
    ftemp1 = 13.522 + count;
    if (count == 5)
    {
        WdtIconSetBlinkParameters (panel[Panel].icones[P12_I1], IMG_GRUE_GD_BLKD, 1);
        WdtIconSetBlinkOnOff (panel[Panel].icones[P12_I1], 1);
    }
    if ((ftemp1_sv != ftemp1))
#else
    if ((GetItemValueByKey(key, &ftemp1) != -1) && (ftemp1_sv != ftemp1))
#endif
    {
      sprintf(item, "%.1f", ftemp1);
      WdtTextFieldSetText(panel[Panel].textfields[P12_T0], item);
      ftemp1_sv = ftemp1;
    }
#ifdef SIMU
    ftemp2 = 177.857 + count;
    if ((ftemp2_sv != ftemp2))
#else
    if ((GetItemValueByKey(key, &ftemp2) != -1) && (ftemp2_sv != ftemp2))
#endif
    {
      sprintf(item, "%.1f", ftemp2);
      WdtTextFieldSetText(panel[Panel].textfields[P12_T1], item);
      ftemp2_sv = ftemp2;
    }
#ifndef SIMU
    if (GetItemValueByKey(key, &ftemp1) != -1)
    {
      sprintf(item, "%.1f", ftemp1);
      WdtLabelSetString(panel[Panel].labels[P12_L3], item);
    }
#endif
#ifdef SIMU
//    ftemp1 = 1;
#else
    GetItemValueByKey(key, &ftemp1);
#endif
    sprintf(item, "%.1f", ftemp1);
    WdtLabelSetString(panel[Panel].labels[P12_L3], item); // Valeur brute
    ftemp1 = ftemp1 * P1X_pente + P1X_offset;
    sprintf(item, "%.1f", ftemp1);
    WdtLabelSetString(panel[Panel].labels[P12_L0], item); // Valeur mise a l'echelle
}

void WdtCheckTOR(int Panel, int icone, int image, int key)
{
    float TOR_test;

    if (GetItemValueByKey(key, &TOR_test) != -1)
    {
      if ((TOR_test == 1.0) && (!WdtIconGetBlinkOnOff (panel[Panel].icones[icone])))
      {
        WdtIconSetBlinkParameters (panel[Panel].icones[icone], image, 1);
        WdtIconSetBlinkOnOff (panel[Panel].icones[icone], 1);
        WdtIconSetTagValue (panel[Panel].icones[icone], key);
//        printf("TOR %d is ON\n", key);
      }
      if ((TOR_test == 0.0) && (WdtIconGetBlinkOnOff (panel[Panel].icones[icone])))
        if (key == WdtIconGetTagValue(panel[Panel].icones[icone])) 
        {
          WdtIconSetBlinkOnOff (panel[Panel].icones[icone], 0);
//          printf("TOR %d is OFF\n", key);
        }
    }
}

int  WdtPanelGetSelected () {
  return selected_panel;
}

int WdtPanelSetSelected (int aPanel) {
  if (aPanel >= MAX_PANELS) { 
    printf ("ActivatePanel : Panel %d is greater than %d\n", aPanel, MAX_PANELS); 
    return 1;// KO 
  } 
  selected_panel = aPanel;

  WdtPanelDraw (); // DRAW IT !

  return 0; // OK
}

// Be carreful. We only enter this function when a key is pressed.
// In this case you are able to manage states ONLY when a key goes down.
void WdtPanelManage (int key, int key_level) 
{
  FSM_STATE_ID aState;
  int          status;
  char         temp[20];
  int          temp_val;
  float        ftemp_val;
  SplOim_t    *OIMconf;
  unsigned int piddlz;
  FILE        *fp;

  if (key_level == Wdt_KEY_UP) return;

  if (WdtFsmGetNextState (key, &aState) == 0) return; // No change we leave

  switch (aState) {
    // --> FB10
  case FSM_P2:
    WdtPanelSetSelected (PANEL_P2);
    WdtFrameSetVisibility (panel[PANEL_P2].frames[P3_F13], 0);
    WdtIconSetSelection (panel[PANEL_P2].icones[P2_I0], 0);
    break;
    
  case FSM_P2_S1:
    WdtIconSetSelection (panel[PANEL_P2].icones[P2_I0], 1);
    WdtFrameSetVisibility (panel[PANEL_P2].frames[P2_F13], 0);
    break;
    
  case FSM_P2_S2:
    WdtIconSetSelection (panel[PANEL_P2].icones[P2_I0], 0);
    WdtFrameSetVisibility (panel[PANEL_P2].frames[P2_F13], 1);
    break;
    
  case FSM_P2_NAVIG:
    if (key == Wdt_KEY_LEFT) {
      if (current_P2_sheet > 0) {
        current_P2_sheet--;
        WdtPanelChangeValues_P2 ();
      }      
    }
    else if (key == Wdt_KEY_RIGHT) {
      int nbcycles;
      GetNbCycles (&nbcycles);
      if (current_P2_sheet < nbcycles-1) {
	current_P2_sheet++;
	WdtPanelChangeValues_P2 ();
      }
    }
    else if (key == Wdt_KEY_ESC) {
      WdtIconSetSelection (panel[PANEL_P2].icones[P2_I0], 0);
      WdtFrameSetVisibility (panel[PANEL_P2].frames[P2_F13], 0);
      //current_P2_sheet = 0;
      //WdtPanelChangeValues_P2 ();
      WdtFsmSetState (FSM_P2);
    }
  break;
  // <-- FB10  
   
  case FSM_A0:
    WdtPanelSetSelected (PANEL_A0);
    WdtTextFieldSetText (panel[PANEL_A0].textfields[0], "12.34"); // *****
    WdtFsmSetState (FSM_A0_Entry);
    break;

  case FSM_P3: // P2->P3
    WdtPanelSetSelected (PANEL_P3);
    WdtFrameSetVisibility (panel[PANEL_P3].frames[P3_F21], 0);
    //current_P3_sheet = 0;
    //WdtPanelChangeValues_P3 ();
    break;

  case FSM_P3_S1:
    WdtIconSetSelection (panel[PANEL_P3].icones[P3_I0], 1);
    WdtFrameSetVisibility (panel[PANEL_P3].frames[P3_F21], 0);
    break;

  case FSM_P3_S2:
    WdtIconSetSelection (panel[PANEL_P3].icones[P3_I0], 0);
    WdtFrameSetVisibility (panel[PANEL_P3].frames[P3_F21], 1);
    break;

  case FSM_P3_NAVIG:
    if (key == Wdt_KEY_LEFT) 
    {
      if (current_P3_sheet > 0) 
      {
        current_P3_sheet--;
        WdtPanelChangeValues_P3 ();
      }      
    }
    else 
    if (key == Wdt_KEY_RIGHT) 
    {
      int nblog;
      GetNbLogs (&nblog);
      if (current_P3_sheet < ((nblog + 9)/ 10) - 1) 
      {
        current_P3_sheet++;
        WdtPanelChangeValues_P3 ();
      }
    }
    else if (key == Wdt_KEY_ESC) 
    {
      WdtIconSetSelection (panel[PANEL_P3].icones[P3_I0], 0);
      WdtFrameSetVisibility (panel[PANEL_P3].frames[P3_F21], 0);
      current_P3_sheet = 0;
      WdtPanelChangeValues_P3 ();
      WdtFsmSetState (FSM_P3);
    }
    break;

  case FSM_P1:
    WdtIconSetSelection (panel[PANEL_P1].icones[P1_I0], 0); 
    WdtIconSetSelection (panel[PANEL_P1].icones[P1_I5], 0); 
    WdtIconSetSelection (panel[PANEL_P1].icones[P1_I6], 0); 
    WdtIconSetSelection (panel[PANEL_P1].icones[P1_I7], 0); 
    WdtIconSetSelection (panel[PANEL_P1].icones[P1_I8], 0); 
    WdtPanelSetSelected (PANEL_P1);
    break;

  case FSM_A0_Entry: // MESSAGE BOX
    status = WdtTextFieldManage(panel[PANEL_A0].textfields[0], key);
    if (status == 2) { // TextField has been validated
      if (strcmp (WdtTextFieldGetText (panel[PANEL_A0].textfields[0]), "31542") == 0) 
      {
        printf ("Yes ! You found the password\n");
        WdtFsmSetState (FSM_S1);
        WdtIconSetSelection (panel[PANEL_P1].icones[P1_I0], 1);
        WdtPanelSetSelected (PANEL_P1);
      }
      else {
        printf ("Wrong password. try 31542 and see if it is better !\n");
        WdtFsmSetState (FSM_P1);
        WdtPanelSetSelected (PANEL_P1);
      }
    }
    else if (status == 1) { // TextField has been left (ESC key)
      WdtPanelSetSelected (PANEL_P1);
      WdtFsmSetState (FSM_P1);
    }
    break;

  case FSM_A1:
    WdtPanelSetSelected (PANEL_A1);
    break;

  case FSM_A2:
    WdtPanelSetSelected (PANEL_A2);
    break;

  case FSM_S1:
    WdtIconSetSelection (panel[PANEL_P1].icones[P1_I0], 1); // S1
    WdtIconSetSelection (panel[PANEL_P1].icones[P1_I8], 0); // !S5
    WdtIconSetSelection (panel[PANEL_P1].icones[P1_I5], 0); // !S2
    break;

  case FSM_S2:
    if (WdtCheckBoxGetChecked (panel[PANEL_P11].checkboxes[P11_C0]))
    {
      WdtIconSetSelection (panel[PANEL_P1].icones[P1_I0], 0); // !S1
      WdtIconSetSelection (panel[PANEL_P1].icones[P1_I6], 0); // !S3
      WdtIconSetSelection (panel[PANEL_P1].icones[P1_I5], 1); // S2
      break;
    }
    else
    {
      WdtIconSetSelection (panel[PANEL_P1].icones[P1_I0], 0); // !S1
      WdtIconSetSelection (panel[PANEL_P1].icones[P1_I6], 0); // !S3
      WdtFsmSetState (FSM_S3);
      WdtPanelSetSelected (PANEL_P1);
      break;
    }
  
  case FSM_S3:
    if (WdtCheckBoxGetChecked (panel[PANEL_P11].checkboxes[P11_C1]))
    {
      WdtIconSetSelection (panel[PANEL_P1].icones[P1_I7], 0); // !S4
      WdtIconSetSelection (panel[PANEL_P1].icones[P1_I5], 0); // !S2
      WdtIconSetSelection (panel[PANEL_P1].icones[P1_I6], 1); // S3
      break;
    }
    else
    {
      WdtIconSetSelection (panel[PANEL_P1].icones[P1_I7], 0); // !S1
      WdtIconSetSelection (panel[PANEL_P1].icones[P1_I5], 0); // !S3
      WdtFsmSetState (FSM_S4);
    }

  case FSM_S4:
    if (WdtCheckBoxGetChecked (panel[PANEL_P11].checkboxes[P11_C2]))
    {
      WdtIconSetSelection (panel[PANEL_P1].icones[P1_I8], 0); // !S5
      WdtIconSetSelection (panel[PANEL_P1].icones[P1_I6], 0); // !S3
      WdtIconSetSelection (panel[PANEL_P1].icones[P1_I7], 1); // S4
      break;
    }
    else
    {
      WdtIconSetSelection (panel[PANEL_P1].icones[P1_I8], 0); // !S5
      WdtIconSetSelection (panel[PANEL_P1].icones[P1_I6], 0); // !S3
      WdtFsmSetState (FSM_S4);
    }

  case FSM_S5:
    if (WdtCheckBoxGetChecked (panel[PANEL_P11].checkboxes[P11_C3]))
    {
      WdtIconSetSelection (panel[PANEL_P1].icones[P1_I7], 0); // !S4
      WdtIconSetSelection (panel[PANEL_P1].icones[P1_I8], 1); // S5
      break;
    }
    else
    {
      WdtIconSetSelection (panel[PANEL_P1].icones[P1_I7], 0); // !S4
      WdtFsmSetState (FSM_S1);
      WdtPanelSetSelected (PANEL_P1);
    }
    break;

  case FSM_P11:
    WdtCheckBoxSetFocus (panel[PANEL_P11].checkboxes[P11_C0], 1);
    WdtCheckBoxSetFocus (panel[PANEL_P11].checkboxes[P11_C1], 0);
    WdtCheckBoxSetFocus (panel[PANEL_P11].checkboxes[P11_C2], 0);
    WdtCheckBoxSetFocus (panel[PANEL_P11].checkboxes[P11_C3], 0);
    WdtPanelSetSelected (PANEL_P11);
    break;

  case FSM_P11_C0:
    WdtCheckBoxSetFocus (panel[PANEL_P11].checkboxes[P11_C0], 1);
    WdtCheckBoxSetFocus (panel[PANEL_P11].checkboxes[P11_C3], 0);
    WdtCheckBoxSetFocus (panel[PANEL_P11].checkboxes[P11_C1], 0);
    break;

  case FSM_P11_C1:
    WdtCheckBoxSetFocus (panel[PANEL_P11].checkboxes[P11_C1], 1);
    WdtCheckBoxSetFocus (panel[PANEL_P11].checkboxes[P11_C0], 0);
    WdtCheckBoxSetFocus (panel[PANEL_P11].checkboxes[P11_C2], 0);
    break;

  case FSM_P11_C2:
    WdtCheckBoxSetFocus (panel[PANEL_P11].checkboxes[P11_C2], 1);
    WdtCheckBoxSetFocus (panel[PANEL_P11].checkboxes[P11_C1], 0);
    WdtCheckBoxSetFocus (panel[PANEL_P11].checkboxes[P11_C3], 0);
    break;

  case FSM_P11_C3:
    WdtCheckBoxSetFocus (panel[PANEL_P11].checkboxes[P11_C3], 1);
    WdtCheckBoxSetFocus (panel[PANEL_P11].checkboxes[P11_C2], 0);
    WdtCheckBoxSetFocus (panel[PANEL_P11].checkboxes[P11_C0], 0);
    break;

  case FSM_P11_C0_ENT:
    WdtCheckBoxSetChecked (panel[PANEL_P11].checkboxes[P11_C0],  
                           !WdtCheckBoxGetChecked (panel[PANEL_P11].checkboxes[P11_C0]));
    Compute_P1_BLOCK();
    break;

  case FSM_P11_C1_ENT:
    WdtCheckBoxSetChecked (panel[PANEL_P11].checkboxes[P11_C1],
                           !WdtCheckBoxGetChecked (panel[PANEL_P11].checkboxes[P11_C1]));
    Compute_P1_BLOCK();
    break;

  case FSM_P11_C2_ENT:
    WdtCheckBoxSetChecked (panel[PANEL_P11].checkboxes[P11_C2],
                           !WdtCheckBoxGetChecked (panel[PANEL_P11].checkboxes[P11_C2]));
    Compute_P1_BLOCK();
    break;

  case FSM_P11_C3_ENT:
    WdtCheckBoxSetChecked (panel[PANEL_P11].checkboxes[P11_C3],
                           !WdtCheckBoxGetChecked (panel[PANEL_P11].checkboxes[P11_C3]));
    Compute_P1_BLOCK();
    break;

  case FSM_P12:
    RecupConfigSplOim();
    OIMconf    = GetOimFromTypePhy(TP_PORTEE_DIST);
    P1X_pente  = OIMconf->Pente;
    P1X_offset = OIMconf->Offset;
    printf("P1X_pente = %f, P1X_offset = %f\n", P1X_pente, P1X_offset);
    WdtTextFieldSetFrozen (panel[PANEL_P12].textfields[P12_T0], 0);
    WdtTextFieldSetFrozen (panel[PANEL_P12].textfields[P12_T1], 0);
    WdtPanelSetSelected (PANEL_P12);
    break;

  case FSM_P13:
    WdtPanelSetSelected (PANEL_P13);
    break;

  case FSM_P14:
    WdtPanelSetSelected (PANEL_P14);
    break;

  case FSM_P15:
    WdtPanelSetSelected (PANEL_P15);
    break;

  case FSM_S33:
    WdtIconSetSelection (panel[PANEL_P11].icones[P11_I20], 0); // S34
    WdtIconSetSelection (panel[PANEL_P11].icones[P11_I22], 0); // S32
    WdtIconSetSelection (panel[PANEL_P11].icones[P11_I21], 1); // S33
    break;

  case FSM_S31:
    WdtIconSetSelection (panel[PANEL_P11].icones[P11_I20], 0); // !S4
    WdtIconSetSelection (panel[PANEL_P11].icones[P11_I22], 0); 
    WdtIconSetSelection (panel[PANEL_P11].icones[P11_I23], 1); // S31
    break;
 
  case FSM_S32:
    WdtIconSetSelection (panel[PANEL_P11].icones[P11_I21], 0);
    WdtIconSetSelection (panel[PANEL_P11].icones[P11_I23], 0); 
    WdtIconSetSelection (panel[PANEL_P11].icones[P11_I22], 1); 
    break;
      
  case FSM_S34:
    WdtIconSetSelection (panel[PANEL_P11].icones[P11_I21], 0); 
    WdtIconSetSelection (panel[PANEL_P11].icones[P11_I23], 0); 
    WdtIconSetSelection (panel[PANEL_P11].icones[P11_I20], 1); 
    break;
    
  case FSM_P21:
    WdtPanelSetSelected (PANEL_P21);
    break;

  case FSM_P22:
    WdtPanelSetSelected (PANEL_P22);
    break;

  case FSM_P12_1:
    WdtTextFieldSetFocus(panel[PANEL_P12].textfields[P12_T2], 1);
    WdtIconSetSelection (panel[PANEL_P12].icones[P12_I2], 0);
    WdtIconSetSelection (panel[PANEL_P12].icones[P12_I6], 0);
    break;

  case FSM_P12_1_Entry:
    if (WdtTextFieldManage (panel[PANEL_P12].textfields[P12_T2], key) == 2)
    {
      sscanf(WdtTextFieldGetText(panel[PANEL_P12].textfields[P12_T2]), "%d", &temp_val);
//      printf("Value entered = %d\n", temp_val);
      P1X_val3 = temp_val;
      WdtTextFieldSetFocus(panel[PANEL_P12].textfields[P12_T2], 0);
      WdtIconSetSelection (panel[PANEL_P12].icones[P12_I6], 1);
      WdtFsmSetState (FSM_P12_2);
      WdtPanelSetSelected (PANEL_P12);
    }
    else 
    {
      if (key==Wdt_KEY_ESC) 
      {
        WdtPanelSetSelected (PANEL_P12);
        WdtFsmSetState (FSM_P12_1);
      }
    }
    break;

  case FSM_P12_2:
    WdtTextFieldSetFocus(panel[PANEL_P12].textfields[P12_T2], 0);
    WdtTextFieldSetFocus(panel[PANEL_P12].textfields[P12_T3], 0);
    WdtIconSetSelection (panel[PANEL_P12].icones[P12_I6], 1);
    break;

  case FSM_P12_2_Entry:
    WdtTextFieldSetFrozen (panel[PANEL_P12].textfields[P12_T0], 1);
    sscanf(WdtTextFieldGetText(panel[PANEL_P12].textfields[P12_T0]), "%f", &ftemp_val);
    P1X_val1   = ftemp_val;
    P1X_pente  = (P1X_val4 - P1X_val3) / (P1X_val2 - P1X_val1);
    P1X_offset =  P1X_val3 - (P1X_val1 * P1X_pente);
    printf("P1X_val1 = %f, P1X_val2 = %f\n P1X_val3 = %f, P1X_val4 = %f\n", P1X_val1, P1X_val2, P1X_val3, P1X_val4);
    printf("pente = %f, offset = %f\n", P1X_pente, P1X_offset);
    break;

  case FSM_P12_3:
    WdtTextFieldSetFocus(panel[PANEL_P12].textfields[P12_T3], 1);
    WdtIconSetSelection (panel[PANEL_P12].icones[P12_I6], 0);
    WdtIconSetSelection (panel[PANEL_P12].icones[P12_I7], 0);
    break;

  case FSM_P12_3_Entry:
    if (WdtTextFieldManage (panel[PANEL_P12].textfields[P12_T3], key) == 2)
    {
      sscanf(WdtTextFieldGetText(panel[PANEL_P12].textfields[P12_T3]), "%d", &temp_val);
//      printf("Value entered = %d\n", temp_val);
      P1X_val4 = temp_val;
      WdtTextFieldSetFocus(panel[PANEL_P12].textfields[P12_T3], 0);
      WdtIconSetSelection (panel[PANEL_P12].icones[P12_I7], 1);
      WdtFsmSetState (FSM_P12_4);
      WdtPanelSetSelected (PANEL_P12);
    }
    else
    {
      if (key==Wdt_KEY_ESC)
      {
        WdtPanelSetSelected (PANEL_P12);
        WdtFsmSetState (FSM_P12_1);
      }
    }
    break;

  case FSM_P12_4:
    WdtTextFieldSetFocus(panel[PANEL_P12].textfields[P12_T3], 0);
    WdtIconSetSelection (panel[PANEL_P12].icones[P12_I7], 1);
    WdtIconSetSelection (panel[PANEL_P12].icones[P12_I2], 0);
    break;

  case FSM_P12_4_Entry:
    WdtTextFieldSetFrozen (panel[PANEL_P12].textfields[P12_T1], 1);
    sscanf(WdtTextFieldGetText(panel[PANEL_P12].textfields[P12_T1]), "%f", &ftemp_val);
    P1X_val2   = ftemp_val;
    P1X_pente  = (P1X_val4 - P1X_val3) / (P1X_val2 - P1X_val1);
    P1X_offset =  P1X_val3 - (P1X_val1 * P1X_pente);
    printf("P1X_val1 = %f, P1X_val2 = %f\n P1X_val3 = %f, P1X_val4 = %f\n", P1X_val1, P1X_val2, P1X_val3, P1X_val4);
    printf("pente = %f, offset = %f\n", P1X_pente, P1X_offset);
    break;

  case FSM_P12_5:
    WdtIconSetSelection (panel[PANEL_P12].icones[P12_I7], 0);
    WdtIconSetSelection (panel[PANEL_P12].icones[P12_I2], 1);
    WdtTextFieldSetFocus(panel[PANEL_P12].textfields[P12_T2], 0);
    break;

 case FSM_P12_5_Entry:
    RecupConfigSplOim();
    OIMconf    = GetOimFromTypePhy(TP_PORTEE_DIST);
    OIMconf->Pente  = P1X_pente;
    OIMconf->Offset = P1X_offset;
#ifndef SIMU
    if (mount("/dev/hda2", "/mnt/config", "ext2", MS_REMOUNT|MS_SYNCHRONOUS, "") < 0)
    {
      printf("Erreur de remontage en RW de /mnt/confg\n");
      break;
    }
#endif
    WriteConfigOim();
#ifndef SIMU
    if (mount("/dev/hda2", "/mnt/config", "ext2", MS_REMOUNT|MS_SYNCHRONOUS|MS_RDONLY, "") < 0)
    {
      printf("Erreur de remontage en RW de /mnt/confg\n");
      break;
    }
    fp = fopen("/var/dlzbin.pid", "r");
    fscanf(fp, "pid=%d", &piddlz);
    fclose(fp);
    if (kill(piddlz, 1) < 0)
      printf("Erreur d'envoi du signal SIGHUP au process %d\n", piddlz);
#endif
    printf("Fichier %s sauvegarde !\n", LIBOIM_CONFIG_FILE_NAME);
    WdtPanelSetSelected (PANEL_P1);
    WdtFsmSetState (FSM_S1);
    break;
 
  default:
    printf ("WdtPanelManage : Unknown state %d\n", aState);
    break;
  }
}

void Compute_P1_BLOCK(void)
{
  int block[P11_C3+1];
  int line[4];
  int checked[4];
  int i,j;
  block[P11_C0] = 1000;
  block[P11_C1] = 1000;
  block[P11_C2] = 1000;
  block[P11_C3] = 1000;
  line[0] = BLOCK1;
  line[1] = BLOCK2;
  line[2] = BLOCK3;
  line[3] = BLOCK4;

  for (i = 0; i < 4; i++)
    checked[i] = WdtCheckBoxGetChecked (panel[PANEL_P11].checkboxes[i]);

  for (i = 0; i < 4; i++)
  {
    for (j = P11_C0; j <= P11_C3; j++)
    {
      if (WdtCheckBoxGetChecked (panel[PANEL_P11].checkboxes[j]))
      {
        WdtCheckBoxSetChecked (panel[PANEL_P11].checkboxes[j], 0);
        block[j] = line[i];
        break;
      }
    }
  }

  panel[PANEL_P1].icones[P1_I5]->y          = block[P11_C0];
  panel[PANEL_P1].frames[P1_F0]->y          = block[P11_C0]+80;
  panel[PANEL_P1].icones[P1_I9]->y          = block[P11_C0]+32;
  panel[PANEL_P1].progress[P1_P0]->y        = block[P11_C0]+50;
  panel[PANEL_P1].progress[P1_P0]->label->y = block[P11_C0]+55;

  panel[PANEL_P1].icones[P1_I6]->y          = block[P11_C1];
  panel[PANEL_P1].frames[P1_F1]->y          = block[P11_C1]+80;
  panel[PANEL_P1].icones[P1_I10]->y         = block[P11_C1]+32;
  panel[PANEL_P1].progress[P1_P1]->y        = block[P11_C1]+50;
  panel[PANEL_P1].progress[P1_P1]->label->y = block[P11_C1]+55;

  panel[PANEL_P1].icones[P1_I7]->y          = block[P11_C2];
  panel[PANEL_P1].frames[P1_F2]->y          = block[P11_C2]+80;
  panel[PANEL_P1].icones[P1_I11]->y         = block[P11_C2]+16;
  panel[PANEL_P1].progress[P1_P2]->y        = block[P11_C2]+50;
  panel[PANEL_P1].progress[P1_P2]->label->y = block[P11_C2]+55;

  panel[PANEL_P1].icones[P1_I8]->y          = block[P11_C3];
  panel[PANEL_P1].frames[P1_F3]->y          = block[P11_C3]+80;
  panel[PANEL_P1].icones[P1_I12]->y         = block[P11_C3]+18;
  panel[PANEL_P1].progress[P1_P3]->y        = block[P11_C3]+50;
  panel[PANEL_P1].progress[P1_P3]->label->y = block[P11_C3]+55;

  for (i = 0; i < 4; i++)
    WdtCheckBoxSetChecked (panel[PANEL_P11].checkboxes[i], checked[i]);
}

//
// Panel P1
//
void WdtPanelInit_P1 (GR_DRAW_ID wid, GR_GC_ID gc, GR_GC_ID gc_progress) 
{
  Wdt_PROGRESS *p0, *p1, *p2, *p3;
  float chariot, levage, moment, charge; 
  IhmConfig_t Wdtconf;
  SplOim_t   *OIMconf;

  GetOimValueByType(OIM_LEVAGE, &levage);
  GetOimValueByType(OIM_CHARIOT, &chariot);
  GetOimValueByType(OIM_MOMENT, &moment);
  GetOimValueByType(OIM_CHARGE, &charge);
  printf("levage = %f, chariot = %f, moment = %f, charge = %f\n", levage, chariot, moment, charge);

  // UPPER AREA
  time(&tm);
  memcpy(&tm_time, localtime(&tm), sizeof(tm_time));
  sprintf(HM_string, "%02d:%02d", tm_time.tm_hour, tm_time.tm_min);
  sprintf(DATE_string, "%02d/%02d/%02d", tm_time.tm_mday, tm_time.tm_mon+1, tm_time.tm_year-100);
  RecupConfigIhm(&Wdtconf);
  printf("P1 : %s\n", Wdtconf.MarqueGrue);
  sprintf(machine, "%s - %s", Wdtconf.MarqueGrue, Wdtconf.ModeleGrue);
  sprintf(ID, "%s", Wdtconf.NumSerieGrue);
  FreeIhmMem(&Wdtconf);
  panel[PANEL_P1].icones[P1_I0] = WdtIconNew (wid, gc, 10, 10, IMG_FEUILLE_P1_BLANCHE, IMG_FEUILLE_P1_NOIRE);
  panel[PANEL_P1].icones[P1_I1] = WdtIconNew (wid, gc, 300, 10, IMG_HORLOGE, -1);
  panel[PANEL_P1].labels[P1_L0] = WdtLabelNew (wid, gc, 80, 30, machine);
  panel[PANEL_P1].labels[P1_L1] = WdtLabelNew (wid, gc, 80, 50, ID);
  panel[PANEL_P1].labels[P1_L2] = WdtLabelNew (wid, gc, 350, 40, HM_string);
  panel[PANEL_P1].labels[P1_L3] = WdtLabelNew (wid, gc, 400, 40, DATE_string);

  RecupConfigSplOim();
  // First BLOCK
  panel[PANEL_P1].icones[P1_I5] = WdtIconNew (wid, gc, 20, BLOCK1, IMG_GRUE_GD, IMG_GRUE_GD_NOIRE);
  panel[PANEL_P1].frames[P1_F0] = WdtFrameNew (wid, gc, 10, BLOCK1+80, 460, 2);
  panel[PANEL_P1].icones[P1_I9]  = WdtIconNew (wid, gc, 410, BLOCK1+32, IMG_UNITE_M, -1);
  OIMconf = GetOimFromTypePhy(TP_PORTEE_DIST);
  p0 = WdtProgressNew (wid, gc, gc_progress, 130, BLOCK1+50, 260, 18, Wdt_PROGRESS_HORIZONTAL, OIMconf->ValMin, OIMconf->ValMax, 1);
  WdtProgressSetValue (p0, chariot);
  panel[PANEL_P1].progress[P1_P0] = p0;

//  WdtIconSetBlinkParameters (panel[PANEL_P1].icones[P1_I5], IMG_GRUE_GD_BLKD, 1);
//  WdtIconSetBlinkOnOff (panel[PANEL_P1].icones[P1_I5], 1);

  // SECOND BLOCK
  panel[PANEL_P1].icones[P1_I6] = WdtIconNew (wid, gc, 20, BLOCK2, IMG_GRUE_HB, IMG_GRUE_HB_NOIRE);
  panel[PANEL_P1].frames[P1_F1] = WdtFrameNew (wid, gc, 10, BLOCK2+80, 460, 2);
  panel[PANEL_P1].icones[P1_I10] = WdtIconNew (wid, gc, 410, BLOCK2+32, IMG_UNITE_M, -1);
  OIMconf = GetOimFromTypePhy(TP_LEVAGE_DIST);
  p1 = WdtProgressNew (wid, gc, gc_progress, 130, BLOCK2+50, 260, 18, Wdt_PROGRESS_HORIZONTAL, OIMconf->ValMin, OIMconf->ValMax, 1);
  WdtProgressSetValue (p1, levage);
  panel[PANEL_P1].progress[P1_P1] = p1;
  // THIRD BLOCK
  panel[PANEL_P1].icones[P1_I7] = WdtIconNew (wid, gc, 20, BLOCK3, IMG_GRUE_ELEVATOR, IMG_GRUE_ELEVATOR_NOIRE);
  panel[PANEL_P1].frames[P1_F2] = WdtFrameNew (wid, gc, 10, BLOCK3+80, 460, 2);
  panel[PANEL_P1].icones[P1_I11] = WdtIconNew (wid, gc, 410, BLOCK3+16, IMG_UNITE_POURCENT, -1);
  OIMconf = GetOimFromTypePhy(TP_MOMENT);
  p2 = WdtProgressNew (wid, gc, gc_progress, 130, BLOCK3+50, 260, 18, Wdt_PROGRESS_HORIZONTAL, OIMconf->ValMin, OIMconf->ValMax, 0);
  WdtProgressSetValue (p2, moment);
  panel[PANEL_P1].progress[P1_P2] = p2;
  // FOURTH BLOCK
  panel[PANEL_P1].icones[P1_I8] = WdtIconNew (wid, gc, 20, BLOCK4, IMG_GRUE_POIDS, IMG_GRUE_POIDS_NOIRE);
  panel[PANEL_P1].frames[P1_F3] = WdtFrameNew (wid, gc, 10, BLOCK4+80, 460, 2);
  panel[PANEL_P1].icones[P1_I12] = WdtIconNew (wid, gc, 410, BLOCK4+18, IMG_UNITE_T, -1);
  OIMconf = GetOimFromTypePhy(TP_CHARGE);
  p3 = WdtProgressNew (wid, gc, gc_progress, 130, BLOCK4+50, 260, 18, Wdt_PROGRESS_HORIZONTAL, OIMconf->ValMin/1000, OIMconf->ValMax/1000, 2);
  WdtProgressSetValue (p3, charge);
  panel[PANEL_P1].progress[P1_P3] = p3;

  // BOTTOM AREA
//  panel[PANEL_P1].icones[P1_I2] = WdtIconNew (wid, gc, 20, 580, IMG_ANEMOMETRE, -1);
  panel[PANEL_P1].icones[P1_I3] = WdtIconNew (wid, gc, 420, 590, IMG_PALANT_PETIT, -1);
  panel[PANEL_P1].icones[P1_I4] = WdtIconNew (wid, gc, 440, 600, IMG_FOIS2_GROS, -1);
//  panel[PANEL_P1].labels[P1_L4] = WdtLabelNew (wid, gc, 60, 620, WindSpeed);
}


#define  MAINFRM_X         50
#define  MAINFRM_Y        400
#define  MAINFRM_WIDTH    215
#define  MAINFRM_HEIGHT   160
#define  LINE_X            20
#define  LINE_Y           100
#define  LINE_WIDTH       446
#define  LINE_Y_GAP        40
#define  PROGRESS_X       350
#define  PROGRESS_Y       365
//
// Panel P2
//
void WdtPanelInit_P2 (GR_DRAW_ID wid, GR_GC_ID gc, GR_GC_ID gc_progress)
{
  Wdt_PROGRESS *p0;
  unsigned int frame, yline, icone, result;

  // GENERIC AREA
  time(&tm);
  memcpy(&tm_time, localtime(&tm), sizeof(tm_time));
  sprintf(HM_string, "%02d:%02d", tm_time.tm_hour, tm_time.tm_min);
  sprintf(DATE_string, "%02d/%02d/%02d", tm_time.tm_mday, tm_time.tm_mon+1, tm_time.tm_year-100);
  panel[PANEL_P2].labels[P2_L0] = WdtLabelNew (wid, gc, 80, 30, machine);
  panel[PANEL_P2].labels[P2_L1] = WdtLabelNew (wid, gc, 80, 50, ID);
  panel[PANEL_P2].labels[P2_L2] = WdtLabelNew (wid, gc, 350, 40, HM_string);
  panel[PANEL_P2].labels[P2_L3] = WdtLabelNew (wid, gc, 400, 40, DATE_string);
//  panel[PANEL_P2].labels[P2_L4] = WdtLabelNew (wid, gc, 60, 620, WindSpeed);
  panel[PANEL_P2].icones[P2_I0] = WdtIconNew (wid, gc, 10, 10, IMG_FEUILLE_P2_BLANCHE, IMG_FEUILLE_P2_NOIRE);
  panel[PANEL_P2].icones[P2_I1] = WdtIconNew (wid, gc, 300, 10, IMG_HORLOGE, -1);
//  panel[PANEL_P2].icones[P2_I2] = WdtIconNew (wid, gc, 20, 580, IMG_ANEMOMETRE, -1);
  panel[PANEL_P2].icones[P2_I3] = WdtIconNew (wid, gc, 420, 590, IMG_PALANT_PETIT, -1);
  panel[PANEL_P2].icones[P2_I4] = WdtIconNew (wid, gc, 440, 600, IMG_FOIS2_GROS, -1);


  // PROGRESS AREA
  p0 = WdtProgressNew (wid, gc, gc_progress, PROGRESS_X+80, PROGRESS_Y, 18, 200, Wdt_PROGRESS_VERTICAL, 0, 100, -1);
  WdtProgressSetValue (p0, 0);
  panel[PANEL_P2].progress[P2_P0] = p0;
  panel[PANEL_P2].frames[P2_F19]= WdtFrameNew (wid, gc, PROGRESS_X, PROGRESS_Y+10, 60, 21);
  panel[PANEL_P2].frames[P2_F20]= WdtFrameNew (wid, gc, PROGRESS_X, PROGRESS_Y+63, 60, 21);
  panel[PANEL_P2].frames[P2_F21]= WdtFrameNew (wid, gc, PROGRESS_X, PROGRESS_Y+116, 60, 21);
  panel[PANEL_P2].frames[P2_F22]= WdtFrameNew (wid, gc, PROGRESS_X, PROGRESS_Y+169, 60, 21);
  panel[PANEL_P2].labels[P2_NbCyclesZ1] = WdtLabelNew (wid, gc, PROGRESS_X+10, PROGRESS_Y+26, ""); // FB10
  panel[PANEL_P2].labels[P2_NbCyclesZ2] = WdtLabelNew (wid, gc, PROGRESS_X+10, PROGRESS_Y+79, "");// FB10
  panel[PANEL_P2].labels[P2_NbCyclesZ3] = WdtLabelNew (wid, gc, PROGRESS_X+10, PROGRESS_Y+132, "");// FB10
  panel[PANEL_P2].labels[P2_NbCyclesZ4] = WdtLabelNew (wid, gc, PROGRESS_X+10, PROGRESS_Y+185, "");// FB10

  panel[PANEL_P2].frames[P2_F23]= WdtFrameNew (wid, gc, PROGRESS_X+70, PROGRESS_Y+50, 40, 2);
  panel[PANEL_P2].frames[P2_F24]= WdtFrameNew (wid, gc, PROGRESS_X+70, PROGRESS_Y+100, 40, 2);
  panel[PANEL_P2].frames[P2_F25]= WdtFrameNew (wid, gc, PROGRESS_X+70, PROGRESS_Y+150, 40, 2);
  panel[PANEL_P2].labels[P2_L9] = WdtLabelNew (wid, gc, PROGRESS_X+30, PROGRESS_Y+55, "100%");
  panel[PANEL_P2].labels[P2_L10] = WdtLabelNew (wid, gc, PROGRESS_X+30, PROGRESS_Y+105, " 75%");
  panel[PANEL_P2].labels[P2_L11] = WdtLabelNew (wid, gc, PROGRESS_X+30, PROGRESS_Y+155, " 50%");

  // MAIN FRAME AREA
  panel[PANEL_P2].frames[P2_F0] = WdtFrameNew (wid, gc, MAINFRM_X-1, MAINFRM_Y-1, MAINFRM_WIDTH+2, MAINFRM_HEIGHT+2); // central frame
  panel[PANEL_P2].frames[P2_F1] = WdtFrameNew (wid, gc, MAINFRM_X, MAINFRM_Y, MAINFRM_WIDTH, MAINFRM_HEIGHT); // central frame
  panel[PANEL_P2].frames[P2_F2] = WdtFrameNew (wid, gc, MAINFRM_X+8, MAINFRM_Y+40, MAINFRM_WIDTH-16, 1); // H line 1
  panel[PANEL_P2].frames[P2_F3] = WdtFrameNew (wid, gc, MAINFRM_X+8, MAINFRM_Y+80, MAINFRM_WIDTH-16, 1); // H line 2 
  panel[PANEL_P2].frames[P2_F4] = WdtFrameNew (wid, gc, MAINFRM_X+8, MAINFRM_Y+120,MAINFRM_WIDTH-16, 1); // H line 3
  panel[PANEL_P2].frames[P2_F5] = WdtFrameNew (wid, gc, MAINFRM_X+(MAINFRM_WIDTH/2), MAINFRM_Y+8, 1, MAINFRM_HEIGHT-16); // V line
  panel[PANEL_P2].icones[P2_I6] = WdtIconNew (wid, gc, MAINFRM_X-20, MAINFRM_Y-34, IMG_PAILLASSE, -1);

  // --> FB10
  panel[PANEL_P2].frames[P2_F13_1]= WdtFrameNew (wid, gc, MAINFRM_X+31, MAINFRM_Y-34, 58, 28);
  panel[PANEL_P2].frames[P2_F13_2]= WdtFrameNew (wid, gc, MAINFRM_X+29, MAINFRM_Y-36, 62, 32);
  panel[PANEL_P2].frames[P2_F13]  = WdtFrameNew (wid, gc, MAINFRM_X+30, MAINFRM_Y-35, 60, 30);
  WdtFrameSetVisibility (panel[PANEL_P2].frames[P2_F13], 0);
  panel[PANEL_P2].labels[P2_Cycles] = WdtLabelNew (wid, gc, MAINFRM_X+44, MAINFRM_Y-12, "");

  panel[PANEL_P2].labels[P2_DateTimeDepart] = WdtLabelNew (wid, gc, MAINFRM_X+96, MAINFRM_Y-6, "");
  panel[PANEL_P2].labels[P2_Duree] = WdtLabelNew (wid, gc, MAINFRM_X+10, MAINFRM_Y+26, "");
  panel[PANEL_P2].icones[P2_Moufflage] = WdtIconNew (wid, gc, MAINFRM_X+(MAINFRM_WIDTH/2)+10, MAINFRM_Y+12, -1, -1);

  panel[PANEL_P2].labels[P2_Moment] = WdtLabelNew (wid, gc, MAINFRM_X+10, MAINFRM_Y+66, "");
  panel[PANEL_P2].labels[P2_Charge] = WdtLabelNew (wid, gc, MAINFRM_X+10, MAINFRM_Y+106, "");
  panel[PANEL_P2].labels[P2_PosCharDep] = WdtLabelNew (wid, gc, MAINFRM_X+10, MAINFRM_Y+146, "");
  panel[PANEL_P2].labels[P2_PosCharArr] = WdtLabelNew (wid, gc, MAINFRM_X+(MAINFRM_WIDTH/2)+10, MAINFRM_Y+146, "");
  // <-- FB10

  // LINES AREA
  panel[PANEL_P2].icones[P2_I5] = WdtIconNew (wid, gc, 426, 54, IMG_DISQUETTE, -1);
  panel[PANEL_P2].labels[P2_DateRAZ] = WdtLabelNew (wid, gc, 10, 93, "");

  panel[PANEL_P2].frames[P2_F6] = WdtFrameNew (wid, gc, LINE_X-10, LINE_Y, LINE_WIDTH+10, 2);
  panel[PANEL_P2].frames[P2_F7] = WdtFrameNew (wid, gc, LINE_X-10, LINE_Y+2, LINE_WIDTH+10, 2);
  yline = LINE_Y + LINE_Y_GAP;
  // Line1
  panel[PANEL_P2].frames[P2_F8] = WdtFrameNew (wid, gc, LINE_X, yline, LINE_WIDTH, 1);
  panel[PANEL_P2].icones[P2_I7] = WdtIconNew  (wid, gc, LINE_X-5, yline-40, IMG_HORLOGE, -1);
  panel[PANEL_P2].frames[P2_F14]= WdtFrameNew (wid, gc, LINE_X+LINE_WIDTH-80, yline-25, 80, 21);
  panel[PANEL_P2].labels[P2_TempsTotTrav] = WdtLabelNew (wid, gc, LINE_X+LINE_WIDTH-70, yline-8, ""); // FB10
  yline += LINE_Y_GAP;
  // Line2
  panel[PANEL_P2].frames[P2_F9] = WdtFrameNew (wid, gc, LINE_X, yline, LINE_WIDTH, 1);
  panel[PANEL_P2].icones[P2_I8] = WdtIconNew  (wid, gc, LINE_X, yline-30, IMG_FLECHE_GD, -1);
  panel[PANEL_P2].frames[P2_F15]= WdtFrameNew (wid, gc, LINE_X+LINE_WIDTH-80, yline-25, 80, 21);
  panel[PANEL_P2].labels[P2_TempsChariot] = WdtLabelNew (wid, gc, LINE_X+LINE_WIDTH-70, yline-8, "");// FB10
  yline += LINE_Y_GAP;
  // Line3
  panel[PANEL_P2].frames[P2_F10]= WdtFrameNew (wid, gc, LINE_X, yline, LINE_WIDTH, 1);
  panel[PANEL_P2].icones[P2_I9] = WdtIconNew  (wid, gc, LINE_X, yline-32, IMG_FLECHE_HB, -1);
  panel[PANEL_P2].frames[P2_F16]= WdtFrameNew (wid, gc, LINE_X+LINE_WIDTH-80, yline-25, 80, 21);
  panel[PANEL_P2].labels[P2_TempsLevage] = WdtLabelNew (wid, gc,LINE_X+LINE_WIDTH-70, yline-8, "");// FB10
  yline += LINE_Y_GAP;
  // Line4
  panel[PANEL_P2].frames[P2_F11]= WdtFrameNew (wid, gc, LINE_X, yline, LINE_WIDTH, 1);
  panel[PANEL_P2].icones[P2_I10]= WdtIconNew  (wid, gc, LINE_X, yline-30, IMG_FLECHE_ELEVATOR, -1);
  panel[PANEL_P2].frames[P2_F17]= WdtFrameNew (wid, gc, LINE_X+LINE_WIDTH-80, yline-25, 80, 21);
 panel[PANEL_P2].labels[P2_TempsElevator] = WdtLabelNew (wid, gc, LINE_X+LINE_WIDTH-70, yline-8, "");// FB10
  yline += LINE_Y_GAP;
  // Line5
  panel[PANEL_P2].frames[P2_F12]= WdtFrameNew (wid, gc, LINE_X, yline, LINE_WIDTH, 1);
  panel[PANEL_P2].icones[P2_I11]= WdtIconNew (wid, gc, LINE_X, yline-34, IMG_POIDS, -1);
  panel[PANEL_P2].frames[P2_F18]= WdtFrameNew (wid, gc, LINE_X+LINE_WIDTH-80, yline-25, 80, 21);
  panel[PANEL_P2].labels[P2_SommeCharges] = WdtLabelNew (wid, gc, LINE_X+LINE_WIDTH-70, yline-8, "");// FB10

  WdtPanelChangeValues_P2 (); // FB10
}

#define TABLE_X                 30
#define TABLE_Y                 LINE_Y+30
#define TABLE_WIDTH             480-TABLE_X-TABLE_X
#define TABLE_COL1              TABLE_X
#define TABLE_COL2              TABLE_X+150 //TABLE_X+(TABLE_WIDTH/3)
#define TABLE_COL3              TABLE_X+300 //TABLE_X+(TABLE_WIDTH/3*2)
#define TABLE_HEIGHT            400 //640-TABLE_Y-80
#define TABLE_Y_GAP             LINE_Y_GAP
//
// Panel P3
//
void WdtPanelInit_P3 (GR_DRAW_ID wid, GR_GC_ID gc, GR_GC_ID gc_progress)
{
  unsigned int frame, yline, icone, label;
//  int current_P3_sheet = 0;

  // GENERIC AREA
  time(&tm);
  memcpy(&tm_time, localtime(&tm), sizeof(tm_time));
  sprintf(HM_string, "%02d:%02d", tm_time.tm_hour, tm_time.tm_min);
  sprintf(DATE_string, "%02d/%02d/%02d", tm_time.tm_mday, tm_time.tm_mon+1, tm_time.tm_year-100);
  panel[PANEL_P3].labels[P3_L0] = WdtLabelNew (wid, gc, 80, 30, machine);
  panel[PANEL_P3].labels[P3_L1] = WdtLabelNew (wid, gc, 80, 50, ID);
  panel[PANEL_P3].labels[P3_L2] = WdtLabelNew (wid, gc, 350, 40, HM_string);
  panel[PANEL_P3].labels[P3_L3] = WdtLabelNew (wid, gc, 400, 40, DATE_string);
//  panel[PANEL_P3].labels[P3_L4] = WdtLabelNew (wid, gc, 60, 620, WindSpeed);
  panel[PANEL_P3].icones[P3_I0] = WdtIconNew (wid, gc, 10, 10, IMG_FEUILLE_P3_BLANCHE, IMG_FEUILLE_P3_NOIRE);
  panel[PANEL_P3].icones[P3_I1] = WdtIconNew (wid, gc, 300, 10, IMG_HORLOGE, -1);
//  panel[PANEL_P3].icones[P3_I2] = WdtIconNew (wid, gc, 20, 580, IMG_ANEMOMETRE, -1);
  panel[PANEL_P3].icones[P3_I3] = WdtIconNew (wid, gc, 420, 590, IMG_PALANT_PETIT, -1);
  panel[PANEL_P3].icones[P3_I4] = WdtIconNew (wid, gc, 440, 600, IMG_FOIS2_GROS, -1);

  // LINES AREA
  panel[PANEL_P3].icones[P3_I5] = WdtIconNew (wid, gc, 426, 54, IMG_PAILLASSE, -1);
  panel[PANEL_P3].labels[P3_L5] = WdtLabelNew (wid, gc, 330, 93, "");
  panel[PANEL_P3].frames[P3_F19]= WdtFrameNew (wid, gc, 315, 72, 60, 24);
  panel[PANEL_P3].frames[P3_F20]= WdtFrameNew (wid, gc, 313, 70, 64, 28);
  panel[PANEL_P3].frames[P3_F21]= WdtFrameNew (wid, gc, 314, 71, 62, 26);
  WdtFrameSetVisibility (panel[PANEL_P3].frames[P3_F21], 0);
  panel[PANEL_P3].frames[P3_F0] = WdtFrameNew (wid, gc, LINE_X-10, LINE_Y, LINE_WIDTH+10, 2); // top line
  panel[PANEL_P3].frames[P3_F1] = WdtFrameNew (wid, gc, LINE_X-10, LINE_Y+2, LINE_WIDTH+10, 2); // top line
  panel[PANEL_P3].icones[P3_I6] = WdtIconNew (wid, gc, TABLE_COL1+45, TABLE_Y-5, IMG_HORLOGE, -1);
  panel[PANEL_P3].icones[P3_I7] = WdtIconNew (wid, gc, TABLE_COL2+55, TABLE_Y-5, IMG_BLOCNOTE, -1);
  panel[PANEL_P3].frames[P3_F2] = WdtFrameNew (wid, gc, TABLE_X, TABLE_Y+40, TABLE_WIDTH, 2); // table line
  panel[PANEL_P3].frames[P3_F3] = WdtFrameNew (wid, gc, TABLE_X, TABLE_Y+42, TABLE_WIDTH, 2); // table line
  panel[PANEL_P3].frames[P3_F4] = WdtFrameNew (wid, gc, TABLE_COL2, TABLE_Y+30, 2, TABLE_HEIGHT+10); // table line
  panel[PANEL_P3].frames[P3_F5] = WdtFrameNew (wid, gc, TABLE_COL3, TABLE_Y+30, 2, TABLE_HEIGHT+10); // table line

  yline = TABLE_Y + 40 + TABLE_Y_GAP;
  for (frame = 0; frame < 10; frame++)
  {
    panel[PANEL_P3].icones[frame+P2_I8] = WdtIconNew (wid, gc, TABLE_COL2+10, yline-33, -1, -1);
    panel[PANEL_P3].frames[frame+P2_F6] = WdtFrameNew (wid, gc, TABLE_X, yline, TABLE_WIDTH, 1);
    if (yline > (TABLE_Y+TABLE_HEIGHT))
      break;
    yline += TABLE_Y_GAP;
    icone++;
  } 

  yline = TABLE_Y + 40 + TABLE_Y_GAP;
  for (label=0; label < 10; label++) {
    panel[PANEL_P3].labels[label+P3_L6]  = WdtLabelNew (wid, gc, TABLE_COL1+10, yline-12, "");
    panel[PANEL_P3].labels[label+P3_L17] = WdtLabelNew (wid, gc, TABLE_COL3+10, yline-12, "");
    yline += TABLE_Y_GAP;
//    if (yline > (TABLE_Y+TABLE_HEIGHT))
//      break;
  }
  WdtPanelChangeValues_P3 ();
}

void  WdtPanelChangeValues_P3 () {
  int nbtotallogs;
  int nbpages;
  int nbcurrentlogs;
  int i;
  char buf [100];
  int im_id;

  // get the total number of logs
  if (GetNbLogs(&nbtotallogs) != 0) {
    printf ("GetNbLogs : failed\n", nbtotallogs);
    return;
  }

  // Compute the number of visible log according to the page number
  nbcurrentlogs = nbtotallogs - (current_P3_sheet * 10);
  if (nbcurrentlogs >= 10) nbcurrentlogs = 10;

  // Compute the number of pages
  nbpages = (nbtotallogs + 9) / 10; // 10 -> 1, 11 -> 2, 19->2, 20->2

  // We show the current page numbner and the total number of pages
  sprintf (buf, "%d / %d", current_P3_sheet+1, nbpages);
  WdtLabelSetString (panel[PANEL_P3].labels[P3_L5], buf);

  //GetTabLog(2, 2);

  if ( GetTabLog(current_P3_sheet*10, &nbcurrentlogs) == 0) 
  {
    for (i = 0; i < 10; i++) 
    {
      if (i < nbcurrentlogs) 
      { // Only all we need
        sprintf (buf, "%s", TabLog[i].DateTimeDepart);
        WdtLabelSetString (panel[PANEL_P3].labels[i+P3_L6], buf);

        // 154 : chariot avant / chariot stop    (icon grue_gd a droite)   
        //       [ avant ou stop a determiner en fonction du champ information]
        // 155 : chariot arriere / chariot stop  (icon grue_gd gauche)
        // 156 : levage montee / stop            (icon grue_hb haut)
        // 157 : levage descente / stop          (icon grue_hb bas)
        // 158 : secu moment debut / fin         (icon grue_elevator)
        // 159 : secu charge                     (icon poids)
        // 160 : DLZ ON / OFF                    (icon clef)

        // We show correct ICON (second column) only for the following Ident
        switch (TabLog[i].Ident) 
	{
        case 151: im_id = IMG_FLECHE_HB; break;
        case 152: im_id = IMG_FLECHE_GD; break;
        case 153: im_id = IMG_POIDS; break;
        case 154: im_id = IMG_FLECHE_DROITE; break;
        case 155: im_id = IMG_FLECHE_GAUCHE; break;
        case 156: im_id = IMG_FLECHE_HAUT; break;
        case 157: im_id = IMG_FLECHE_BAS; break;
        case 158: im_id = IMG_FLECHE_ELEVATOR; break;
        case 159: im_id = IMG_POIDS; break;
        case 160: im_id = IMG_CLEF; break;

        default:
          im_id = -1;
          break;
        }
        WdtIconSetImage (panel[PANEL_P3].icones[i+P2_I8], im_id, -1);

        // We show the field Value (third column) only for the following Ident
        switch (TabLog[i].Ident) 
	{
        case 151:
        case 152:
        case 153:
          sprintf (buf, "%d", TabLog[i].Value);
          break;
        default:
          strcpy (buf, "");
          break;
        }
        WdtLabelSetString (panel[PANEL_P3].labels[i+P3_L17], buf);
      }
      else { // We erase the fields
        strcpy (buf, "");
        WdtLabelSetString (panel[PANEL_P3].labels[i+P3_L16], buf);
        WdtLabelSetString (panel[PANEL_P3].labels[i+P3_L6], buf);
      }
    }
  }
  else {
    printf ("GetTabLog : failed\n");
  }
}

// --> FB10
void  WdtPanelChangeValues_P2 () {
  int nbtotalcycles;
  int cycle;
  
  int i;
  char buf [100];
  int im_id;
  
  if (GetNbCycles(&nbtotalcycles) != 0) {
    printf ("GetNbCycles : failed\n", nbtotalcycles);
    return;
  }

  if (GetPermLogs() != 0) {
    printf ("GetPermLogs : failed\n");
    return;
  }

  cycle = current_P2_sheet;

  
  // DateRAZ
  sprintf (buf, "%s", PermLog.DateRAZ);
  WdtLabelSetString (panel[PANEL_P2].labels[P2_DateRAZ], buf);

  // TempsTotTrav
  sprintf (buf, "%s", PermLog.TempsTotTrav);
  WdtLabelSetString (panel[PANEL_P2].labels[P2_TempsTotTrav], buf);

  // TempsChariot
  sprintf (buf, "%s", PermLog.TempsChariot);
  WdtLabelSetString (panel[PANEL_P2].labels[P2_TempsChariot], buf);

  // TempsLevage
  sprintf (buf, "%s", PermLog.TempsLevage);
  WdtLabelSetString (panel[PANEL_P2].labels[P2_TempsLevage], buf);

  // TempsElevator
  //// sprintf (buf, "%s", PermLog.TempsElevator );
  sprintf (buf, "Moment = 2 !");
  WdtLabelSetString (panel[PANEL_P2].labels[P2_TempsElevator], buf);

  // SommeCharges
  sprintf (buf, "%s", PermLog.SommeCharges);
  WdtLabelSetString (panel[PANEL_P2].labels[P2_SommeCharges], buf);

  // NbCyclesZ1
  sprintf (buf, "%d", PermLog.NbCyclesZ1);
  WdtLabelSetString (panel[PANEL_P2].labels[P2_NbCyclesZ1], buf);

  // NbCyclesZ2
  sprintf (buf, "%d", PermLog.NbCyclesZ2);
  WdtLabelSetString (panel[PANEL_P2].labels[P2_NbCyclesZ2], buf);

  // NbCyclesZ3
  sprintf (buf, "%d", PermLog.NbCyclesZ3);
  WdtLabelSetString (panel[PANEL_P2].labels[P2_NbCyclesZ3], buf);

  // NbCyclesZ4
  sprintf (buf, "%d", PermLog.NbCyclesZ4);
  WdtLabelSetString (panel[PANEL_P2].labels[P2_NbCyclesZ4], buf);

  // Show the 1/ n
  sprintf (buf, "%d / %d", cycle+1, nbtotalcycles);
  WdtLabelSetString (panel[PANEL_P2].labels[P2_Cycles], buf);

#ifdef SIMU
  if ( GetCycle_STUB (cycle) == 0) {
#else
  if ( GetCycle (cycle) == 0) {
#endif
    sprintf (buf, "%s", CycleLog.DateTimeDepart);
    WdtLabelSetString (panel[PANEL_P2].labels[P2_DateTimeDepart], buf);

    sprintf (buf, "%s", CycleLog.Duree);
    WdtLabelSetString (panel[PANEL_P2].labels[P2_Duree], buf);

    sprintf (buf, "%s", CycleLog.Moment);
    WdtLabelSetString (panel[PANEL_P2].labels[P2_Moment], buf);

    sprintf (buf, "%s", CycleLog.Charge);
    WdtLabelSetString (panel[PANEL_P2].labels[P2_Charge], buf);

    sprintf (buf, "%s", CycleLog.PosCharDep);
    WdtLabelSetString (panel[PANEL_P2].labels[P2_PosCharDep], buf);

    sprintf (buf, "%s", CycleLog.PosCharArr);
    WdtLabelSetString (panel[PANEL_P2].labels[P2_PosCharArr], buf);

    // Moufflage
    switch (CycleLog.TypeMoufflage) {
    case 0: im_id=IMG_FOIS2; break;
    case 1: im_id=IMG_FOIS4; break;
    default:im_id=-1;break;
    }

    WdtIconSetImage (panel[PANEL_P2].icones[P2_Moufflage], im_id, -1);
  }
  else {
    printf ("GetCycle (%d): failed\n", cycle);
  }
}
// <-- FB10

//
// Panel A0
//
void WdtPanelInit_A0 (GR_DRAW_ID wid, GR_GC_ID gc, GR_GC_ID gc_progress)
{
  panel[PANEL_A0].frames[A0_F0] = WdtFrameNew (wid, gc, MSGBOX_Y, MSGBOX_X, MSGBOX_WIDTH, MSGBOX_HEIGHT);
  panel[PANEL_A0].frames[A0_F1] = WdtFrameNew (wid, gc, MSGBOX_Y, MSGBOX_X, MSGBOX_WIDTH+1, MSGBOX_HEIGHT+1);
  panel[PANEL_A0].frames[A0_F2] = WdtFrameNew (wid, gc, MSGBOX_Y, MSGBOX_X, MSGBOX_WIDTH+2, MSGBOX_HEIGHT+2);
  panel[PANEL_A0].icones[A0_I0] = WdtIconNew (wid, gc, MSGBOX_Y+50, MSGBOX_X+40, IMG_CADENAS_GRAND, -1);
  // Numeric- 4 digits
  panel[PANEL_A0 ].textfields[0] = WdtTextFieldNew (wid, gc, MSGBOX_Y+130, MSGBOX_X+75, 200, 30, 1, 5);
  WdtTextFieldSetJustify (panel[PANEL_A0].textfields[0], 1);
  WdtTextFieldSetFocus (panel[PANEL_A0].textfields[0],1);


  // --> FB13
  panel[PANEL_A0].labels[0] = WdtLabelNew (wid, gc, MSGBOX_Y+130, MSGBOX_X+260, ".Gauche"); // To the left (default)
  WdtLabelSetJustify (panel[PANEL_A0].labels[0], 0);

  panel[PANEL_A0].labels[1] = WdtLabelNew (wid, gc, MSGBOX_Y+130, MSGBOX_X+280, "Droite."); // To the right
  WdtLabelSetJustify (panel[PANEL_A0].labels[1], 1);

  panel[PANEL_A0].textfields[1] = WdtTextFieldNew (wid, gc, MSGBOX_Y+130, MSGBOX_X+160, 200, 30, 0, 5);// To the left (default)
  WdtTextFieldSetJustify (panel[PANEL_A0].textfields[1], 0);
  WdtTextFieldSetText (panel[PANEL_A0].textfields[1], "Left");

  panel[PANEL_A0].textfields[2] = WdtTextFieldNew (wid, gc, MSGBOX_Y+130, MSGBOX_X+200, 200, 30, 0, 5);// To the right
  WdtTextFieldSetJustify (panel[PANEL_A0].textfields[2], 1);
  WdtTextFieldSetText (panel[PANEL_A0].textfields[2], "Right");
  // <-- FB13
}

//  
// Panel A1  
//
void WdtPanelInit_A1 (GR_DRAW_ID wid, GR_GC_ID gc, GR_GC_ID gc_progress)
{
  panel[PANEL_A1].frames[A1_F0] = WdtFrameNew (wid, gc, MSGBOX_Y, MSGBOX_X, MSGBOX_WIDTH, MSGBOX_HEIGHT);
  panel[PANEL_A1].frames[A1_F1] = WdtFrameNew (wid, gc, MSGBOX_Y, MSGBOX_X, MSGBOX_WIDTH+1, MSGBOX_HEIGHT+1);
  panel[PANEL_A1].frames[A1_F2] = WdtFrameNew (wid, gc, MSGBOX_Y, MSGBOX_X, MSGBOX_WIDTH+2, MSGBOX_HEIGHT+2);
  panel[PANEL_A1].icones[A1_I0] = WdtIconNew (wid, gc, MSGBOX_Y+40, MSGBOX_X+50, IMG_WARNING, -1);
  panel[PANEL_A1].icones[A1_I1] = WdtIconNew (wid, gc, MSGBOX_Y+150, MSGBOX_X+100, IMG_OK, -1);
  panel[PANEL_A1].icones[A1_I2] = WdtIconNew (wid, gc, MSGBOX_Y+270, MSGBOX_X+100, IMG_ANNULER, -1);
  panel[PANEL_A1].labels[A1_L0] = WdtLabelNew (wid, gc, MSGBOX_Y+120, MSGBOX_X+50, "Remettre toutes les valeurs par defaut ?");
}

//
// Panel A2
//
void WdtPanelInit_A2 (GR_DRAW_ID wid, GR_GC_ID gc, GR_GC_ID gc_progress)
{
  panel[PANEL_A2].frames[A2_F0] = WdtFrameNew (wid, gc, MSGBOX_Y, MSGBOX_X, MSGBOX_WIDTH, MSGBOX_HEIGHT);
  panel[PANEL_A2].frames[A2_F1] = WdtFrameNew (wid, gc, MSGBOX_Y, MSGBOX_X, MSGBOX_WIDTH+1, MSGBOX_HEIGHT+1);
  panel[PANEL_A2].frames[A2_F2] = WdtFrameNew (wid, gc, MSGBOX_Y, MSGBOX_X, MSGBOX_WIDTH+2, MSGBOX_HEIGHT+2);
  panel[PANEL_A2].icones[A2_I0] = WdtIconNew (wid, gc, MSGBOX_Y+40, MSGBOX_X+50, IMG_WARNING, -1);
  panel[PANEL_A2].icones[A2_I1] = WdtIconNew (wid, gc, MSGBOX_Y+150, MSGBOX_X+100, IMG_OK, -1);
  panel[PANEL_A2].icones[A2_I2] = WdtIconNew (wid, gc, MSGBOX_Y+270, MSGBOX_X+100, IMG_ANNULER, -1);
  panel[PANEL_A2].labels[A2_L0] = WdtLabelNew (wid, gc, MSGBOX_Y+120, MSGBOX_X+50, "Voulez vous enregistrer les nouveaux");
  panel[PANEL_A2].labels[A2_L1] = WdtLabelNew (wid, gc, MSGBOX_Y+190, MSGBOX_X+65, "parametres ?");
}

//
// Panel A3
//
void WdtPanelInit_A3 (GR_DRAW_ID wid, GR_GC_ID gc, GR_GC_ID gc_progress)
{
  panel[PANEL_A3].frames[A3_F0] = WdtFrameNew (wid, gc, MSGBOX_Y, MSGBOX_X, MSGBOX_WIDTH, MSGBOX_HEIGHT);
  panel[PANEL_A3].frames[A3_F1] = WdtFrameNew (wid, gc, MSGBOX_Y, MSGBOX_X, MSGBOX_WIDTH+1, MSGBOX_HEIGHT+1);
  panel[PANEL_A3].frames[A3_F2] = WdtFrameNew (wid, gc, MSGBOX_Y, MSGBOX_X, MSGBOX_WIDTH+2, MSGBOX_HEIGHT+2);
  panel[PANEL_A3].icones[A3_I0] = WdtIconNew (wid, gc, MSGBOX_Y+40, MSGBOX_X+50, IMG_WARNING, -1);
  panel[PANEL_A3].icones[A3_I1] = WdtIconNew (wid, gc, MSGBOX_Y+150, MSGBOX_X+100, IMG_OK, -1);
  panel[PANEL_A3].icones[A3_I2] = WdtIconNew (wid, gc, MSGBOX_Y+270, MSGBOX_X+100, IMG_ANNULER, -1);
  panel[PANEL_A3].labels[A3_L0] = WdtLabelNew (wid, gc, MSGBOX_Y+120, MSGBOX_X+50, "Attention ! Cette operation effacera tous");
  panel[PANEL_A3].labels[A3_L1] = WdtLabelNew (wid, gc, MSGBOX_Y+190, MSGBOX_X+65, "les enregistrements.");
}

//
// Panel A4
//
void WdtPanelInit_A4 (GR_DRAW_ID wid, GR_GC_ID gc, GR_GC_ID gc_progress)
{
  panel[PANEL_A4].frames[A4_F0] = WdtFrameNew (wid, gc, MSGBOX_Y, MSGBOX_X, MSGBOX_WIDTH, MSGBOX_HEIGHT);
  panel[PANEL_A4].frames[A4_F1] = WdtFrameNew (wid, gc, MSGBOX_Y, MSGBOX_X, MSGBOX_WIDTH+1, MSGBOX_HEIGHT+1);
  panel[PANEL_A4].frames[A4_F2] = WdtFrameNew (wid, gc, MSGBOX_Y, MSGBOX_X, MSGBOX_WIDTH+2, MSGBOX_HEIGHT+2);
  panel[PANEL_A4].icones[A4_I0] = WdtIconNew (wid, gc, MSGBOX_Y+40, MSGBOX_X+50, IMG_WARNING, -1);
  panel[PANEL_A4].icones[A4_I1] = WdtIconNew (wid, gc, MSGBOX_Y+150, MSGBOX_X+100, IMG_OK, -1);
  panel[PANEL_A4].icones[A4_I2] = WdtIconNew (wid, gc, MSGBOX_Y+270, MSGBOX_X+100, IMG_ANNULER, -1);
  panel[PANEL_A4].labels[A4_L0] = WdtLabelNew (wid, gc, MSGBOX_Y+140, MSGBOX_X+50, "Veuillez inserer la carte memoire.");
  panel[PANEL_A4].labels[A4_L1] = WdtLabelNew (wid, gc, MSGBOX_Y+110, MSGBOX_X+65, "Attention ! Les informations sur votre carte");
  panel[PANEL_A4].labels[A4_L2] = WdtLabelNew (wid, gc, MSGBOX_Y+190, MSGBOX_X+80, "memoire seront remplacees.");
}

//
// Panel A5
//
void WdtPanelInit_A5 (GR_DRAW_ID wid, GR_GC_ID gc, GR_GC_ID gc_progress)
{
  panel[PANEL_A5].frames[A5_F0] = WdtFrameNew (wid, gc, MSGBOX_Y, MSGBOX_X, MSGBOX_WIDTH, MSGBOX_HEIGHT);
  panel[PANEL_A5].frames[A5_F1] = WdtFrameNew (wid, gc, MSGBOX_Y, MSGBOX_X, MSGBOX_WIDTH+1, MSGBOX_HEIGHT+1);
  panel[PANEL_A5].frames[A5_F2] = WdtFrameNew (wid, gc, MSGBOX_Y, MSGBOX_X, MSGBOX_WIDTH+2, MSGBOX_HEIGHT+2);
  panel[PANEL_A5].icones[A5_I0] = WdtIconNew (wid, gc, MSGBOX_Y+150, MSGBOX_X+100, IMG_OK, -1);
  panel[PANEL_A5].labels[A5_L0] = WdtLabelNew (wid, gc, MSGBOX_Y+80, MSGBOX_X+65, "Les informations sont enregistres");
}

//
// Panel A6
//
void WdtPanelInit_A6 (GR_DRAW_ID wid, GR_GC_ID gc, GR_GC_ID gc_progress)
{
  panel[PANEL_A6].frames[A6_F0] = WdtFrameNew (wid, gc, MSGBOX_Y, MSGBOX_X, MSGBOX_WIDTH, MSGBOX_HEIGHT);
  panel[PANEL_A6].frames[A6_F1] = WdtFrameNew (wid, gc, MSGBOX_Y, MSGBOX_X, MSGBOX_WIDTH+1, MSGBOX_HEIGHT+1);
  panel[PANEL_A6].frames[A6_F2] = WdtFrameNew (wid, gc, MSGBOX_Y, MSGBOX_X, MSGBOX_WIDTH+2, MSGBOX_HEIGHT+2);
  panel[PANEL_A6].icones[A6_I0] = WdtIconNew (wid, gc, MSGBOX_Y+40, MSGBOX_X+50, IMG_WARNING, -1);
  panel[PANEL_A6].icones[A6_I1] = WdtIconNew (wid, gc, MSGBOX_Y+150, MSGBOX_X+100, IMG_OK, -1);
  panel[PANEL_A6].labels[A6_L0] = WdtLabelNew (wid, gc, MSGBOX_Y+150, MSGBOX_X+65, "Probleme d'enregistrement !");
  panel[PANEL_A6].labels[A6_L1] = WdtLabelNew (wid, gc, MSGBOX_Y+100, MSGBOX_X+80, "Les informations ne sont pas enregistrees");
}

#define TOP_LINE_Y              70
#define MIDDLE_LINE_Y           xx
#define UPPER_X                 10
#define UPPER_Y                 TOP_LINE_Y+10
#define UPPER_WIDTH             480-20
#define LOWER_X                 UPPER_X
#define LOWER_Y                 MIDDLE_LINE_Y+10
//
// Panel P11
//
void WdtPanelInit_P11 (GR_DRAW_ID wid, GR_GC_ID gc, GR_GC_ID gc_progress)
{
  // GENERIC AREA
  time(&tm);
  memcpy(&tm_time, localtime(&tm), sizeof(tm_time));
  sprintf(HM_string, "%02d:%02d", tm_time.tm_hour, tm_time.tm_min);
  sprintf(DATE_string, "%02d/%02d/%02d", tm_time.tm_mday, tm_time.tm_mon+1, tm_time.tm_year-100);
  panel[PANEL_P11].labels[P11_L0] = WdtLabelNew (wid, gc, 350, 40, HM_string);
  panel[PANEL_P11].labels[P11_L1] = WdtLabelNew (wid, gc, 400, 40, DATE_string);
  panel[PANEL_P11].icones[P11_I0] = WdtIconNew (wid, gc, 10, 10, IMG_FEUILLE_P11_BLANCHE, IMG_FEUILLE_P11_NOIRE);
  panel[PANEL_P11].icones[P11_I1] = WdtIconNew (wid, gc, 60, 13, IMG_GLOBE, -1);
  panel[PANEL_P11].icones[P11_I2] = WdtIconNew (wid, gc, 300, 10, IMG_HORLOGE, -1);
  panel[PANEL_P11].frames[P11_F0] = WdtFrameNew (wid, gc, 10, 70, 460, 2);
  panel[PANEL_P11].frames[P11_F1] = WdtFrameNew (wid, gc, 120, 30, 120, 16);
//  panel[PANEL_P11].labels[P11_L2] = WdtLabelNew (wid, gc, 130, 20, "Francais");

  // UPPER AREA
  panel[PANEL_P11].icones[P11_I3] = WdtIconNew  (wid, gc,  10,  80,  IMG_CARRE_INFORMATION, -1);
  panel[PANEL_P11].labels[P11_L3] = WdtLabelNew (wid, gc, 100,  92, "1-");
  panel[PANEL_P11].frames[P11_F2] = WdtFrameNew (wid, gc, 115,  78, 130, 16);
  panel[PANEL_P11].labels[P11_L4] = WdtLabelNew (wid, gc, 100, 110, "2-");
  panel[PANEL_P11].frames[P11_F3] = WdtFrameNew (wid, gc, 115,  96, 130, 16);
  panel[PANEL_P11].labels[P11_L5] = WdtLabelNew (wid, gc, 100, 128, "3-");
  panel[PANEL_P11].frames[P11_F4] = WdtFrameNew (wid, gc, 115, 114, 130, 16);
//  panel[PANEL_P11].labels[P11_L6] = WdtLabelNew (wid, gc, 120,  92, machine);
//  panel[PANEL_P11].labels[P11_L7] = WdtLabelNew (wid, gc, 120, 110, machine);
//  panel[PANEL_P11].labels[P11_L8] = WdtLabelNew (wid, gc, 120, 128, ID);
  panel[PANEL_P11].frames[P11_F5] = WdtFrameNew (wid, gc,  10, 134, 460, 1);
  panel[PANEL_P11].frames[P11_F6] = WdtFrameNew (wid, gc, 250,  80,   1, 290);
  panel[PANEL_P11].icones[P11_I4] = WdtIconNew  (wid, gc, 280,  83,  IMG_HORLOGE, -1);
  panel[PANEL_P11].frames[P11_F7] = WdtFrameNew (wid, gc, 350,  85,  30,  16);
  panel[PANEL_P11].frames[P11_F8] = WdtFrameNew (wid, gc, 390,  85,  30,  16);
  panel[PANEL_P11].frames[P11_F9] = WdtFrameNew (wid, gc, 430,  85,  30,  16);
  panel[PANEL_P11].frames[P11_F10]= WdtFrameNew (wid, gc, 350, 110,  30,  16);
  panel[PANEL_P11].frames[P11_F11]= WdtFrameNew (wid, gc, 390, 110,  30,  16);
  panel[PANEL_P11].labels[P11_L9] = WdtLabelNew (wid, gc, 337, 110, "-");
  panel[PANEL_P11].labels[P11_L10]= WdtLabelNew (wid, gc, 383,  98, "/");
  panel[PANEL_P11].labels[P11_L11]= WdtLabelNew (wid, gc, 423,  98, "/");
  panel[PANEL_P11].labels[P11_L12]= WdtLabelNew (wid, gc, 383, 124, ":");
  panel[PANEL_P11].frames[P11_F12]= WdtFrameNew (wid, gc,  10, 200 , 460,  1);
  panel[PANEL_P11].icones[P11_I5] = WdtIconNew  (wid, gc,  10, 140, IMG_POUTRE_POIDS, -1);
  panel[PANEL_P11].labels[P11_L13]= WdtLabelNew (wid, gc, 108, 174, "-");
  panel[PANEL_P11].frames[P11_F13]= WdtFrameNew (wid, gc, 115, 160,  80, 16);
  panel[PANEL_P11].labels[P11_L14]= WdtLabelNew (wid, gc, 200, 174, "Kg");
  panel[PANEL_P11].icones[P11_I6] = WdtIconNew  (wid, gc, 290, 160, IMG_POIDS, -1);
  panel[PANEL_P11].labels[P11_L15]= WdtLabelNew (wid, gc, 337, 174, "-");
  panel[PANEL_P11].frames[P11_F14]= WdtFrameNew (wid, gc, 344, 160,  80, 16);
  panel[PANEL_P11].labels[P11_L16]= WdtLabelNew (wid, gc, 430, 174, "Kg");
  panel[PANEL_P11].icones[P11_I7] = WdtIconNew  (wid, gc,  10, 210, IMG_POUTRE_COTE, -1);
  panel[PANEL_P11].labels[P11_L17]= WdtLabelNew (wid, gc, 108, 234, "-");
  panel[PANEL_P11].frames[P11_F15]= WdtFrameNew (wid, gc, 115, 220,  80, 16);
  panel[PANEL_P11].labels[P11_L18]= WdtLabelNew (wid, gc, 200, 224, "m");
  panel[PANEL_P11].icones[P11_I8] = WdtIconNew  (wid, gc, 270, 220, IMG_ROULETTES, -1);
  panel[PANEL_P11].icones[P11_I9] = WdtIconNew  (wid, gc, 305, 210, IMG_CROCHET, -1);
  panel[PANEL_P11].labels[P11_L19]= WdtLabelNew (wid, gc, 260, 234, "1");
  panel[PANEL_P11].labels[P11_L20]= WdtLabelNew (wid, gc, 300, 234, "+");
  panel[PANEL_P11].labels[P11_L21]= WdtLabelNew (wid, gc, 337, 234, "-");
  panel[PANEL_P11].frames[P11_F16]= WdtFrameNew (wid, gc, 344, 220,  80, 16);
  panel[PANEL_P11].labels[P11_L22]= WdtLabelNew (wid, gc, 430, 234, "Kg");
  panel[PANEL_P11].frames[P11_F17]= WdtFrameNew (wid, gc,  10, 260 , 460,  1);
  panel[PANEL_P11].icones[P11_I10]= WdtIconNew  (wid, gc,  10, 270, IMG_CADENAS_PETIT, -1);
  panel[PANEL_P11].labels[P11_L23]= WdtLabelNew (wid, gc, 100, 290, "1-");
  panel[PANEL_P11].frames[P11_F18]= WdtFrameNew (wid, gc, 115, 276,  80, 16);
  panel[PANEL_P11].labels[P11_L24]= WdtLabelNew (wid, gc, 100, 310, "2-");
  panel[PANEL_P11].frames[P11_F19]= WdtFrameNew (wid, gc, 115, 296,  80, 16);
  panel[PANEL_P11].icones[P11_I11]= WdtIconNew  (wid, gc, 270, 280, IMG_ROULETTES, -1);
  panel[PANEL_P11].icones[P11_I12]= WdtIconNew  (wid, gc, 305, 270, IMG_CROCHET, -1);
  panel[PANEL_P11].labels[P11_L25]= WdtLabelNew (wid, gc, 260, 294, "2");
  panel[PANEL_P11].labels[P11_L26]= WdtLabelNew (wid, gc, 300, 294, "+");
  panel[PANEL_P11].labels[P11_L27]= WdtLabelNew (wid, gc, 337, 294, "-");
  panel[PANEL_P11].frames[P11_F20]= WdtFrameNew (wid, gc, 344, 280,  80, 16);
  panel[PANEL_P11].labels[P11_L28]= WdtLabelNew (wid, gc, 430, 294, "Kg");
  panel[PANEL_P11].frames[P11_F21]= WdtFrameNew (wid, gc,  10, 320 , 460,  1);
  panel[PANEL_P11].icones[P11_I13]= WdtIconNew  (wid, gc, 270, 325, IMG_ANEMOMETRE, -1);
  panel[PANEL_P11].frames[P11_F22]= WdtFrameNew (wid, gc,  10, 380 , 460,  2);

  // LOWER AREA
  panel[PANEL_P11].icones[P11_I14] = WdtIconNew (wid, gc, 40, 400, IMG_FLECHE_GD, -1);
  panel[PANEL_P11].icones[P11_I15] = WdtIconNew (wid, gc, 40, 440, IMG_FLECHE_HB, -1);
  panel[PANEL_P11].icones[P11_I16] = WdtIconNew (wid, gc, 40, 480, IMG_POIDS, -1);
  panel[PANEL_P11].icones[P11_I17] = WdtIconNew (wid, gc, 40, 520, IMG_FLECHE_ELEVATOR, -1);
  panel[PANEL_P11].checkboxes[P11_C0] = WdtCheckBoxNew (wid, gc, 80, 400, 20);
  panel[PANEL_P11].checkboxes[P11_C1] = WdtCheckBoxNew (wid, gc, 80, 440, 20);
  panel[PANEL_P11].checkboxes[P11_C2] = WdtCheckBoxNew (wid, gc, 80, 480, 20);
  panel[PANEL_P11].checkboxes[P11_C3] = WdtCheckBoxNew (wid, gc, 80, 520, 20);
  WdtCheckBoxSetChecked (panel[PANEL_P11].checkboxes[P11_C0], 1);
  WdtCheckBoxSetChecked (panel[PANEL_P11].checkboxes[P11_C1], 1);
  WdtCheckBoxSetChecked (panel[PANEL_P11].checkboxes[P11_C2], 1);
  WdtCheckBoxSetChecked (panel[PANEL_P11].checkboxes[P11_C3], 1);

  // BOTTOM AREA
  panel[PANEL_P11].icones[P11_I18] = WdtIconNew (wid, gc, 240, 410, IMG_ROULETTES, -1);
  panel[PANEL_P11].icones[P11_I19] = WdtIconNew (wid, gc, 360, 410, IMG_ROULETTES, -1);
  panel[PANEL_P11].icones[P11_I20] = WdtIconNew (wid, gc, 10, 580, IMG_CARRE_FLECHE, IMG_CARRE_FLECHE_NOIRE);
  panel[PANEL_P11].icones[P11_I21] = WdtIconNew (wid, gc, 60, 580, IMG_CARRE_LOUPE, IMG_CARRE_LOUPE_NOIRE);
  panel[PANEL_P11].icones[P11_I22]  = WdtIconNew (wid, gc, 370, 580, IMG_DISQUETTE, IMG_DISQUETTE_NOIRE);
  panel[PANEL_P11].icones[P11_I23] = WdtIconNew (wid, gc, 420, 580, IMG_CARRE_CROIX, IMG_CARRE_CROIX_NOIRE);

  // FRAMES
//  panel[PANEL_P11].frames[P11_F0] = WdtFrameNew (wid, gc, 10, 70, 460, 2);
//  panel[PANEL_P11].frames[P11_F1] = WdtFrameNew (wid, gc, 10, 130, 460, 1);
//  panel[PANEL_P11].frames[P11_F2] = WdtFrameNew (wid, gc, 10, 200, 460, 1);
//  panel[PANEL_P11].frames[P11_F3] = WdtFrameNew (wid, gc, 10, 260, 460, 1);
//  panel[PANEL_P11].frames[P11_F4] = WdtFrameNew (wid, gc, 10, 330, 460, 1);
//  panel[PANEL_P11].frames[P11_F5] = WdtFrameNew (wid, gc, 10, 400, 460, 2);

  // CHECKBOXES
/*
  panel[PANEL_P11].checkboxes[P11_C0] = WdtCheckBoxNew (wid, gc, 160, 485, 20);
  WdtCheckBoxSetFocus (panel[PANEL_P11].checkboxes[P11_C0], 1);

  panel[PANEL_P11].checkboxes[P11_C1] = WdtCheckBoxNew (wid, gc, 380, 485, 20);
  WdtCheckBoxSetChecked (panel[PANEL_P11].checkboxes[P11_C1], 1);

  panel[PANEL_P11].checkboxes[P11_C2] = WdtCheckBoxNew (wid, gc, 160, 525, 20);
  
  panel[PANEL_P11].checkboxes[P11_C3] = WdtCheckBoxNew (wid, gc, 380, 525, 20);
  WdtCheckBoxSetChecked (panel[PANEL_P11].checkboxes[P11_C3], 1);
  WdtCheckBoxSetFrozen (panel[PANEL_P11].checkboxes[P11_C3], 1);
*/
}

#define  MSGBOX_BIGY         MSGBOX_Y
#define  MSGBOX_BIGX         MSGBOX_X - 40
#define  MSGBOX_BIGWIDTH     MSGBOX_WIDTH
#define  MSGBOX_BIGHEIGHT    MSGBOX_HEIGHT + 70
//
// Panel P12
//
void WdtPanelInit_P12 (GR_DRAW_ID wid, GR_GC_ID gc, GR_GC_ID gc_progress)
{
  panel[PANEL_P12].frames[P12_F0] = WdtFrameNew (wid, gc, MSGBOX_BIGY, MSGBOX_BIGX, MSGBOX_BIGWIDTH, MSGBOX_BIGHEIGHT);
  panel[PANEL_P12].frames[P12_F1] = WdtFrameNew (wid, gc, MSGBOX_BIGY, MSGBOX_BIGX, MSGBOX_BIGWIDTH+1, MSGBOX_BIGHEIGHT+1);
  panel[PANEL_P12].frames[P12_F2] = WdtFrameNew (wid, gc, MSGBOX_BIGY, MSGBOX_BIGX, MSGBOX_BIGWIDTH+2, MSGBOX_BIGHEIGHT+2);

  // ICONS
  panel[PANEL_P12].icones[P12_I0] = WdtIconNew (wid, gc, MSGBOX_BIGY+10, MSGBOX_BIGX+10, IMG_FEUILLE_P12_BLANCHE, IMG_FEUILLE_P12_NOIRE);
  panel[PANEL_P12].icones[P12_I1] = WdtIconNew (wid, gc, MSGBOX_BIGY+50, MSGBOX_BIGX+10, IMG_GRUE_GD, -1);
  panel[PANEL_P12].icones[P12_I2] = WdtIconNew (wid, gc, MSGBOX_BIGY+MSGBOX_BIGWIDTH-100, MSGBOX_BIGX+10, IMG_DISQUETTE, IMG_DISQUETTE_NOIRE);
  panel[PANEL_P12].icones[P12_I3] = WdtIconNew (wid, gc, MSGBOX_BIGY+MSGBOX_BIGWIDTH-50, MSGBOX_BIGX+10, IMG_CARRE_CROIX, -1);
  panel[PANEL_P12].icones[P12_I4] = WdtIconNew (wid, gc, MSGBOX_BIGY+80, MSGBOX_BIGX+115, IMG_FLECHE_GD, -1);
  panel[PANEL_P12].icones[P12_I5] = WdtIconNew (wid, gc, MSGBOX_BIGY+285, MSGBOX_BIGX+115, IMG_FLECHE_GD, -1);
  panel[PANEL_P12].icones[P12_I6] = WdtIconNew (wid, gc, MSGBOX_BIGY+MSGBOX_BIGWIDTH/2-50, MSGBOX_BIGX+150, IMG_CARRE_OK, IMG_CARRE_OK_NOIRE);
  panel[PANEL_P12].icones[P12_I7] = WdtIconNew (wid, gc, MSGBOX_BIGY+MSGBOX_BIGWIDTH-50, MSGBOX_BIGX+150, IMG_CARRE_OK, IMG_CARRE_OK_NOIRE);
 

  panel[PANEL_P12].textfields[P12_T0] = WdtTextFieldNew (wid, gc, MSGBOX_BIGY+45, MSGBOX_BIGX+150, 90, 24, 1, 5); 
  panel[PANEL_P12].textfields[P12_T1] = WdtTextFieldNew (wid, gc, MSGBOX_BIGY+MSGBOX_BIGWIDTH/2+45, MSGBOX_BIGX+150, 90, 24, 1, 5);
  panel[PANEL_P12].textfields[P12_T2] = WdtTextFieldNew (wid, gc, MSGBOX_BIGY+45, MSGBOX_BIGX+180, 90, 24, 1, 3);
  panel[PANEL_P12].textfields[P12_T3] = WdtTextFieldNew (wid, gc, MSGBOX_BIGY+MSGBOX_BIGWIDTH/2+45, MSGBOX_BIGX+180, 90, 24, 1, 3); 
  panel[PANEL_P12].frames[P12_F3] = WdtFrameNew (wid, gc, MSGBOX_BIGY+44, MSGBOX_BIGX+179, 92, 26);
  panel[PANEL_P12].frames[P12_F4] = WdtFrameNew (wid, gc, MSGBOX_BIGY+MSGBOX_BIGWIDTH/2+44, MSGBOX_BIGX+179, 92, 26);
  WdtTextFieldSetText(panel[PANEL_P12].textfields[P12_T0], "111.5");
  WdtTextFieldSetText(panel[PANEL_P12].textfields[P12_T1], "222.5");
  WdtTextFieldSetText(panel[PANEL_P12].textfields[P12_T2], "333");
  WdtTextFieldSetText(panel[PANEL_P12].textfields[P12_T3], "444");
  WdtTextFieldSetFocus(panel[PANEL_P12].textfields[P12_T2], 1);

  //LABELS
  panel[PANEL_P12].labels[P12_L0] = WdtLabelNew (wid, gc, MSGBOX_BIGY+140, MSGBOX_BIGX+100, "125.44");
  panel[PANEL_P12].labels[P12_L1] = WdtLabelNew (wid, gc, MSGBOX_BIGY+95, MSGBOX_BIGX+110, "1");
  panel[PANEL_P12].labels[P12_L2] = WdtLabelNew (wid, gc, MSGBOX_BIGY+300, MSGBOX_BIGX+110, "2");
  panel[PANEL_P12].labels[P12_L3] = WdtLabelNew (wid, gc, MSGBOX_BIGY+140, MSGBOX_BIGX+80, "125.44");
}

//
// Panel P13
//
void WdtPanelInit_P13 (GR_DRAW_ID wid, GR_GC_ID gc, GR_GC_ID gc_progress)
{
  panel[PANEL_P13].frames[P13_F0] = WdtFrameNew (wid, gc, MSGBOX_BIGY, MSGBOX_BIGX, MSGBOX_BIGWIDTH, MSGBOX_BIGHEIGHT);
  panel[PANEL_P13].frames[P13_F1] = WdtFrameNew (wid, gc, MSGBOX_BIGY, MSGBOX_BIGX, MSGBOX_BIGWIDTH+1, MSGBOX_BIGHEIGHT+1);
  panel[PANEL_P13].frames[P13_F2] = WdtFrameNew (wid, gc, MSGBOX_BIGY, MSGBOX_BIGX, MSGBOX_BIGWIDTH+2, MSGBOX_BIGHEIGHT+2);

  // ICONS
  panel[PANEL_P13].icones[P13_I0] = WdtIconNew (wid, gc, MSGBOX_BIGY+10, MSGBOX_BIGX+10, IMG_FEUILLE_P13_BLANCHE, IMG_FEUILLE_P13_NOIRE);
  panel[PANEL_P13].icones[P13_I1] = WdtIconNew (wid, gc, MSGBOX_BIGY+50, MSGBOX_BIGX+10, IMG_GRUE_HB, -1);
  panel[PANEL_P13].icones[P13_I2] = WdtIconNew (wid, gc, MSGBOX_BIGY+MSGBOX_BIGWIDTH-100, MSGBOX_BIGX+10, IMG_DISQUETTE, -1);
  panel[PANEL_P13].icones[P13_I3] = WdtIconNew (wid, gc, MSGBOX_BIGY+MSGBOX_BIGWIDTH-50, MSGBOX_BIGX+10, IMG_CARRE_CROIX, -1);
  panel[PANEL_P13].icones[P13_I4] = WdtIconNew (wid, gc, MSGBOX_BIGY+80, MSGBOX_BIGX+115, IMG_FLECHE_HB, -1);
  panel[PANEL_P13].icones[P13_I5] = WdtIconNew (wid, gc, MSGBOX_BIGY+285, MSGBOX_BIGX+115, IMG_FLECHE_HB, -1);
  panel[PANEL_P13].icones[P13_I6] = WdtIconNew (wid, gc, MSGBOX_BIGY+MSGBOX_BIGWIDTH/2-50, MSGBOX_BIGX+150, IMG_CARRE_OK, -1);
  panel[PANEL_P13].icones[P13_I7] = WdtIconNew (wid, gc, MSGBOX_BIGY+MSGBOX_BIGWIDTH-50, MSGBOX_BIGX+150, IMG_CARRE_OK, -1);

  // FRAMES
  panel[PANEL_P13].frames[P13_F3] = WdtFrameNew (wid, gc, MSGBOX_BIGY+45, MSGBOX_BIGX+150, 90, 20);
  panel[PANEL_P13].frames[P13_F4] = WdtFrameNew (wid, gc, MSGBOX_BIGY+MSGBOX_BIGWIDTH/2+45, MSGBOX_BIGX+150, 90, 20);
  panel[PANEL_P13].frames[P13_F5] = WdtFrameNew (wid, gc, MSGBOX_BIGY+45, MSGBOX_BIGX+180, 90, 20);
  panel[PANEL_P13].frames[P13_F6] = WdtFrameNew (wid, gc, MSGBOX_BIGY+MSGBOX_BIGWIDTH/2+45, MSGBOX_BIGX+180, 90, 20);

  //LABELS
  panel[PANEL_P13].labels[P13_L0] = WdtLabelNew (wid, gc, MSGBOX_BIGY+140, MSGBOX_BIGX+80, "125.44");
  panel[PANEL_P13].labels[P13_L1] = WdtLabelNew (wid, gc, MSGBOX_BIGY+95, MSGBOX_BIGX+110, "1");
  panel[PANEL_P13].labels[P13_L2] = WdtLabelNew (wid, gc, MSGBOX_BIGY+300, MSGBOX_BIGX+110, "2");
}

//
// Panel P14
//
void WdtPanelInit_P14 (GR_DRAW_ID wid, GR_GC_ID gc, GR_GC_ID gc_progress)
{
  panel[PANEL_P14].frames[P14_F0] = WdtFrameNew (wid, gc, MSGBOX_BIGY, MSGBOX_BIGX, MSGBOX_BIGWIDTH, MSGBOX_BIGHEIGHT);
  panel[PANEL_P14].frames[P14_F1] = WdtFrameNew (wid, gc, MSGBOX_BIGY, MSGBOX_BIGX, MSGBOX_BIGWIDTH+1, MSGBOX_BIGHEIGHT+1);
  panel[PANEL_P14].frames[P14_F2] = WdtFrameNew (wid, gc, MSGBOX_BIGY, MSGBOX_BIGX, MSGBOX_BIGWIDTH+2, MSGBOX_BIGHEIGHT+2);

  // ICONS
  panel[PANEL_P14].icones[P14_I0] = WdtIconNew (wid, gc, MSGBOX_BIGY+10, MSGBOX_BIGX+10, IMG_FEUILLE_P14_BLANCHE, IMG_FEUILLE_P14_NOIRE);
  panel[PANEL_P14].icones[P14_I1] = WdtIconNew (wid, gc, MSGBOX_BIGY+50, MSGBOX_BIGX+10, IMG_GRUE_ELEVATOR, -1);
  panel[PANEL_P14].icones[P14_I2] = WdtIconNew (wid, gc, MSGBOX_BIGY+MSGBOX_BIGWIDTH-100, MSGBOX_BIGX+10, IMG_DISQUETTE, -1);
  panel[PANEL_P14].icones[P14_I3] = WdtIconNew (wid, gc, MSGBOX_BIGY+MSGBOX_BIGWIDTH-50, MSGBOX_BIGX+10, IMG_CARRE_CROIX, -1);
  panel[PANEL_P14].icones[P14_I4] = WdtIconNew (wid, gc, MSGBOX_BIGY+80, MSGBOX_BIGX+115, IMG_FLECHE_ELEVATOR, -1);
  panel[PANEL_P14].icones[P14_I5] = WdtIconNew (wid, gc, MSGBOX_BIGY+285, MSGBOX_BIGX+115, IMG_FLECHE_ELEVATOR, -1);
  panel[PANEL_P14].icones[P14_I6] = WdtIconNew (wid, gc, MSGBOX_BIGY+MSGBOX_BIGWIDTH/2-50, MSGBOX_BIGX+150, IMG_CARRE_OK, -1);
  panel[PANEL_P14].icones[P14_I7] = WdtIconNew (wid, gc, MSGBOX_BIGY+MSGBOX_BIGWIDTH-50, MSGBOX_BIGX+150, IMG_CARRE_OK, -1);

  // FRAMES
  panel[PANEL_P14].frames[P14_F3] = WdtFrameNew (wid, gc, MSGBOX_BIGY+45, MSGBOX_BIGX+150, 90, 20);
  panel[PANEL_P14].frames[P14_F4] = WdtFrameNew (wid, gc, MSGBOX_BIGY+MSGBOX_BIGWIDTH/2+45, MSGBOX_BIGX+150, 90, 20);
  panel[PANEL_P14].frames[P14_F5] = WdtFrameNew (wid, gc, MSGBOX_BIGY+45, MSGBOX_BIGX+180, 90, 20);
  panel[PANEL_P14].frames[P14_F6] = WdtFrameNew (wid, gc, MSGBOX_BIGY+MSGBOX_BIGWIDTH/2+45, MSGBOX_BIGX+180, 90, 20);

  //LABELS
  panel[PANEL_P14].labels[P14_L0] = WdtLabelNew (wid, gc, MSGBOX_BIGY+140, MSGBOX_BIGX+80, "125.44");
  panel[PANEL_P14].labels[P14_L1] = WdtLabelNew (wid, gc, MSGBOX_BIGY+95, MSGBOX_BIGX+110, "1");
  panel[PANEL_P14].labels[P14_L2] = WdtLabelNew (wid, gc, MSGBOX_BIGY+300, MSGBOX_BIGX+110, "2");
}

//
// Panel P15
//
void WdtPanelInit_P15 (GR_DRAW_ID wid, GR_GC_ID gc, GR_GC_ID gc_progress)
{
  panel[PANEL_P15].frames[P15_F0] = WdtFrameNew (wid, gc, MSGBOX_BIGY, MSGBOX_BIGX, MSGBOX_BIGWIDTH, MSGBOX_BIGHEIGHT);
  panel[PANEL_P15].frames[P15_F1] = WdtFrameNew (wid, gc, MSGBOX_BIGY, MSGBOX_BIGX, MSGBOX_BIGWIDTH+1, MSGBOX_BIGHEIGHT+1);
  panel[PANEL_P15].frames[P15_F2] = WdtFrameNew (wid, gc, MSGBOX_BIGY, MSGBOX_BIGX, MSGBOX_BIGWIDTH+2, MSGBOX_BIGHEIGHT+2);

  // ICONS
  panel[PANEL_P15].icones[P15_I0] = WdtIconNew (wid, gc, MSGBOX_BIGY+10, MSGBOX_BIGX+10, IMG_FEUILLE_P15_BLANCHE, IMG_FEUILLE_P15_NOIRE);
  panel[PANEL_P15].icones[P15_I1] = WdtIconNew (wid, gc, MSGBOX_BIGY+50, MSGBOX_BIGX+10, IMG_GRUE_POIDS, -1);
  panel[PANEL_P15].icones[P15_I2] = WdtIconNew (wid, gc, MSGBOX_BIGY+MSGBOX_BIGWIDTH-100, MSGBOX_BIGX+10, IMG_DISQUETTE, -1);
  panel[PANEL_P15].icones[P15_I3] = WdtIconNew (wid, gc, MSGBOX_BIGY+MSGBOX_BIGWIDTH-50, MSGBOX_BIGX+10, IMG_CARRE_CROIX, -1);
  panel[PANEL_P15].icones[P15_I4] = WdtIconNew (wid, gc, MSGBOX_BIGY+90, MSGBOX_BIGX+115, IMG_POIDS, -1);
  panel[PANEL_P15].icones[P15_I5] = WdtIconNew (wid, gc, MSGBOX_BIGY+295, MSGBOX_BIGX+115, IMG_POIDS, -1);
  panel[PANEL_P15].icones[P15_I6] = WdtIconNew (wid, gc, MSGBOX_BIGY+MSGBOX_BIGWIDTH/2-50, MSGBOX_BIGX+150, IMG_CARRE_OK, -1);
  panel[PANEL_P15].icones[P15_I7] = WdtIconNew (wid, gc, MSGBOX_BIGY+MSGBOX_BIGWIDTH-50, MSGBOX_BIGX+150, IMG_CARRE_OK, -1);

  // FRAMES
  panel[PANEL_P15].frames[P15_F3] = WdtFrameNew (wid, gc, MSGBOX_BIGY+45, MSGBOX_BIGX+150, 90, 20);
  panel[PANEL_P15].frames[P15_F4] = WdtFrameNew (wid, gc, MSGBOX_BIGY+MSGBOX_BIGWIDTH/2+45, MSGBOX_BIGX+150, 90, 20);
  panel[PANEL_P15].frames[P15_F5] = WdtFrameNew (wid, gc, MSGBOX_BIGY+45, MSGBOX_BIGX+180, 90, 20);
  panel[PANEL_P15].frames[P15_F6] = WdtFrameNew (wid, gc, MSGBOX_BIGY+MSGBOX_BIGWIDTH/2+45, MSGBOX_BIGX+180, 90, 20);

  //LABELS
  panel[PANEL_P15].labels[P15_L0] = WdtLabelNew (wid, gc, MSGBOX_BIGY+140, MSGBOX_BIGX+80, "125.44");
  panel[PANEL_P15].labels[P15_L1] = WdtLabelNew (wid, gc, MSGBOX_BIGY+95, MSGBOX_BIGX+110, "1");
  panel[PANEL_P15].labels[P15_L2] = WdtLabelNew (wid, gc, MSGBOX_BIGY+300, MSGBOX_BIGX+110, "2");
}


void WdtPanelInit_P22 (GR_DRAW_ID wid, GR_GC_ID gc, GR_GC_ID gc_progress){
}
	

// For test only
void WdtPanelTestP1 (int sens) {
  int value,value2;
  value = WdtProgressGetValue (panel[PANEL_P1].progress[P1_P2]);
  if (sens == 0) {
    value++;
    if (value >=100) {
      value=100;
    }
  }
  else {
    value--;
    if (value < 0) {
      value = 0;
    } 
  }
  
  WdtProgressSetValue (panel[PANEL_P1].progress[P1_P2], value);
  value = WdtProgressGetValue (panel[PANEL_P2].progress[0]);
  if (sens == 0) {
    value++;
    if (value >=100) {
      value=100;
    }
  }
  else {
    value--;
    if (value < 0) {
      value = 0;
    } 
  }
  
  WdtProgressSetValue (panel[PANEL_P2].progress[0], value);
}
// End of file
