/*
 *  Copyright (C) 2008 Florent Bedoiseau (electronique.fb@free.fr)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include "WdtProgress.h"

Wdt_PROGRESS * WdtProgressNew (UInt16 x, UInt16 y, UInt16 w, UInt16 h, UInt8 fgcolor, UInt8 bgcolor, bool orientation, float min, float max, UInt8 decimales) {
  Wdt_PROGRESS *p = (Wdt_PROGRESS *) calloc (1, sizeof (Wdt_PROGRESS));
  if (p) {
    p->x = x;
    p->y = y;
    p->w = w;
    p->h = h;
	p->_fgcolor = fgcolor;
	p->_bgcolor = bgcolor;

    p->validity = true;
    p->orientation = orientation;
    p->min = min;
    p->max = max;
    p->value = min;
    p->decimales=decimales;
    p->visibility = true;
    p->toredraw = true;
    p->label = WdtLabelNew (x, y, fgcolor, bgcolor,"");
  }
  return p;
}

void WdtProgressDelete (Wdt_PROGRESS *aWdtProgress) {
  if (!aWdtProgress) {
#ifdef DEBUG
    printf ("WdtProgressDelete : WdtProgress = NULL\n");
#endif
    return;
  }

  WdtLabelDelete (aWdtProgress->label);
  free (aWdtProgress);
  aWdtProgress = 0;
}

void WdtProgressDraw (Wdt_PROGRESS *aWdtProgress, bool force) {
  UInt16 off;
  Int8 nbdec;
  bool valid;
  float value;
  char buf [20];

  if (!aWdtProgress) {
#ifdef DEBUG
    printf ("WdtProgressDraw : WdtProgress NULL\n");
#endif
    return;
  }

  if (aWdtProgress->visibility == false) return; // Nothing to do

  if ((force == false) && (aWdtProgress->toredraw == false)) // Conditionned by toredraw flag
    return; // Nothing to do

  nbdec = aWdtProgress->decimales;
  valid = aWdtProgress->validity;

  // Draw the "value" first
  if (nbdec != -1) { 
    if (!valid) {
      sprintf (buf, " ----");
    }
    else  {
      value = aWdtProgress->value;
	  if (nbdec == 0) sprintf (buf, "%.0f", value);
	  else if (nbdec == 1) sprintf (buf, "%.1f", value);
	  else if (nbdec == 2) sprintf (buf, "%.2f", value);
	  else sprintf(buf, "%.3f", value);
#if 0
      if (value < 10) {
        if (nbdec == 0) sprintf (buf, "     %.0f", value);
		else if (nbdec == 1) sprintf (buf, "     %.1f", value);
		else sprintf (buf, "     %.2f", value);
      }
      if ((value >= 10) && (value < 100)) {
		if (nbdec == 0) sprintf (buf, "   %.0f", value);
		else if (nbdec == 1) sprintf (buf, "   %.1f", value);
		else sprintf (buf, "   %.2f", value);
      }
      if ((value >= 100) && (value < 1000)) {
        if (nbdec == 0) sprintf (buf, " %.0f", value);
		else if (nbdec == 1) sprintf (buf, " %.1f", value);
		else sprintf (buf, " %.2f", value);
      } 
#endif
    }
    WdtLabelSetString (aWdtProgress->label, buf);
    WdtLabelDraw (aWdtProgress->label, force);
  }

  if (!valid)
    return;

  // Erase the area then draw the border
  GfxFillRectangle (aWdtProgress->x,
					aWdtProgress->y,
					aWdtProgress->w,
					aWdtProgress->h, 
					BGCOLOR);

  GfxRectangle (aWdtProgress->x,
				aWdtProgress->y,
				aWdtProgress->w,
				aWdtProgress->h,
				FGCOLOR);

  if (aWdtProgress->orientation == Wdt_PROGRESS_HORIZONTAL) {

    GfxFillRectangle (aWdtProgress->x + 1,
					  aWdtProgress->y + 1,
					  (UInt16) ((aWdtProgress->w * (aWdtProgress->value-aWdtProgress->min)) / (aWdtProgress->max - aWdtProgress->min) -1),
					  aWdtProgress->h - 2,
					  FGCOLOR);

#ifdef DEBUG
    printf ("HORIZ : W = %d\n", (aWdtProgress->w * (aWdtProgress->value-aWdtProgress->min)) / (aWdtProgress->max - aWdtProgress->min));
    printf ("HORIZ : w:%d, value:%d, max:%d, min:%d\n", aWdtProgress->w, aWdtProgress->value, aWdtProgress->max, aWdtProgress->min);
#endif
  } 
  else {
    off = (UInt16) ((aWdtProgress->h * (aWdtProgress->value-aWdtProgress->min)) / (aWdtProgress->max - aWdtProgress->min));

    GfxFillRectangle (aWdtProgress->x + 1,
					  aWdtProgress->y + aWdtProgress->h - off,
					  aWdtProgress->w - 2,
					  off,
					  FGCOLOR);


#ifdef DEBUG
    printf ("VERT : H = %d\n", (aWdtProgress->h * (aWdtProgress->value-aWdtProgress->min)) / (aWdtProgress->max - aWdtProgress->min));
    printf ("HORIZ : h:%d, value:%d, max:%d, min:%d\n", aWdtProgress->h, aWdtProgress->value, aWdtProgress->max, aWdtProgress->min);
#endif
  }

  aWdtProgress->toredraw = false; // Done
}

float WdtProgressGetValue (Wdt_PROGRESS *aWdtProgress) {
  if (!aWdtProgress) {
#ifdef DEBUG
    printf ("WdtProgressGetValue : WdtProgress NULL\n");
#endif
    return 0.0;
  }
  return aWdtProgress->value;
}

float WdtProgressSetValue (Wdt_PROGRESS *aWdtProgress, float value) {
  if (!aWdtProgress) {
#ifdef DEBUG
    printf ("WdtProgressSetValue : WdtProgress NULL\n");
#endif
    return 0.0;
  }

  if (value == aWdtProgress->value) return value; // Nothing to do : same value

  if (value <= aWdtProgress->min) {
    aWdtProgress->value = aWdtProgress->min;
  }
  else if (value >= aWdtProgress->max) {
    aWdtProgress->value = aWdtProgress->max;
  }
  else {
    aWdtProgress->value = value;
  }

  aWdtProgress->toredraw = true; // Value has changed. Need to redraw
  return aWdtProgress->value;
}

bool WdtProgressGetVisibility (Wdt_PROGRESS *aWdtProgress) {
  if (!aWdtProgress) {
#ifdef DEBUG
    printf ("WdtProgressGetVisibility : WdtProgress : NULL\n");
#endif
    return false;
  }

  return aWdtProgress->visibility;
}

void WdtProgressSetVisibility (Wdt_PROGRESS *aWdtProgress, bool visibility) {
  if (!aWdtProgress) {
#ifdef DEBUG
    printf ("WdtProgressSetVisibility : WdtProgress : NULL\n");
#endif
    return;
  }
  
  if (aWdtProgress->visibility == visibility) return; // Nothing to do
  
  aWdtProgress->visibility = visibility;
  aWdtProgress->toredraw = true; // Visible or not. Need to redraw
}

void WdtProgressSetValidity (Wdt_PROGRESS *aWdtProgress, bool validity) {
  if (!aWdtProgress) {
#ifdef DEBUG
    printf ("WdtProgressSetValidity : WdtProgress : NULL\n");
#endif
    return;
  }

  if (aWdtProgress->validity == validity) return; // Nothing to do

  aWdtProgress->validity = validity;
  aWdtProgress->toredraw = true; // Visible or not. Need to redraw
}

// End of file
